/*
 * Copyright (C) 2006-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package com.funambol.mailclient.ui.controller;

import com.funambol.mailclient.loc.Localization;
import com.funambol.util.Log;

import java.util.Date;
import javax.microedition.midlet.MIDlet;
import javax.microedition.midlet.MIDletStateChangeException;


/**
 * Controller inn charge of LifeCycle activites for the MIDlet
 */
public class LifeCycleController implements Runnable {
    
    private MIDlet midlet;

    public static boolean isPaused;
    
    private static LifeCycleController lifeCycleController;
    
    private LifeCycleController() {
        midlet = UIController.midlet;
    }
    
    public static LifeCycleController getInstance() {
        if (lifeCycleController==null) {
            lifeCycleController = new LifeCycleController();
        }
        return lifeCycleController;
    }
    
    public void pause() {
        Log.info("Application Paused");
        //save status etc.
        isPaused=true;
    }
    
    public void resume() {
        Log.info("Application Resumed");
        // what happens resuming the UI?
        // take time showing a wait screen
        UIController.updateInboxMessageList(null);
    }
    
    /**
     * This method manages the exit of the MIDlet, doing all the
     * jobs related to the OTA messages and alarms before the actual exit
     */
    public void exit() {
        
        UIController.getInboxMessageList().saveSnapshot();
        UIController.getThreadPool().stop();
        try {
            AlarmManager alarmManager = AlarmManager.getInstance();
            if (UIController.getConfig().isSchedulerEnabled()) {
                int pollInterval = UIController.getConfig().getPollInterval();
                if (pollInterval >= 1440) {
                    alarmManager.setAlarm(
                        UIController.getConfig().getNextTimeAlarm());
                } else {
                    long millisecs = pollInterval*60000;
                    Date now = new Date();
                    alarmManager.setAlarm(now.getTime() + millisecs);
                }
                alarmManager.scheduleMIDletAlarm();
            }
            /*if (!(UIController.getConfig().isSmsListenerEnabled())) {
                OTAMessagesManager.getInstance().unregisterForSANMessage();
            } else {
                OTAMessagesManager.getInstance().registerForSANMessage();
            }*/
        } catch (SecurityException ex) {
            Log.error("Exit: " + ex.toString());
            UIController.showAlert(
                Localization.getMessages().ALERT_SECURITY_EXCEPTION_MESSAGE);
            ex.printStackTrace();

        } catch (Exception ex) {
            Log.error("Exit: " + ex.toString());
            UIController.showAlert(
                Localization.getMessages().GENERIC_ERROR_MESSAGE);
            ex.printStackTrace();

        } finally {
            try {
                ((Killable) midlet).exit();
            } catch (MIDletStateChangeException ex) {
                Log.error("Exit: " + ex.toString());
                ex.printStackTrace();
            }
        }
    }
    
    public void run() {
        exit();
    }
    
}
