/*
 * Copyright (C) 2006-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package com.funambol.mailclient.ui.controller;

import com.funambol.util.Log;
import java.io.IOException;
import java.io.InputStream;
import javax.microedition.media.Manager;
import javax.microedition.media.MediaException;
import javax.microedition.media.Player;


/**
 * Class in charge of playing a sound for message received
 */
public class MessageMediaManager implements Runnable {
    InputStream is;
    /** Creates a new instance of MessageMediaManager */
    public MessageMediaManager(InputStream is) {
        this.is = is;
    }
    
    /**
     * Plays a sound (midi file from resources) for message received
     */
    public void playSound() {
        Log.info("play sound...");
        Player player = null;
        try {
            player = Manager.createPlayer(is, "audio/midi");
            player.realize();
            Log.info("start sound...");
            player.start();
            Thread.sleep(2000);
        } catch (IOException ex) {
            Log.error(this, "IOException in playSound");
            ex.printStackTrace();
        } catch (MediaException ex) {
            Log.error(this, "MediaException in playSound");
            ex.printStackTrace();
        } catch (InterruptedException ex) {
            Log.error(this, "InterruptedException in playSound");
            ex.printStackTrace();
        } finally {
            player.close();
            player = null;
        }
        
    }

    /**
     * Run it in separate thread if needed
     */
    public void run() {
        playSound();
    }
}
