/*
 * Copyright (C) 2006-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package com.funambol.mailclient.ui.controller;

import com.funambol.mailclient.Cache;
import com.funambol.mailclient.Funambol;
import com.funambol.mailclient.cm.AlreadyExistentContactException;
import com.funambol.mailclient.cm.ContactManagerFactory;
import com.funambol.mailclient.sm.SyncClient;
import com.funambol.mailclient.ui.utils.UiUtils;
import com.funambol.mailclient.ui.view.AboutScreen;
import com.funambol.mailclient.ui.view.AccountSettingsForm;
import com.funambol.mailclient.ui.view.AddContactForm;
import com.funambol.mailclient.ui.view.AdvancedContactForm;
import com.funambol.mailclient.ui.view.ComposeMessageForm;
import com.funambol.mailclient.ui.view.FolderList;
import com.funambol.mailclient.ui.view.FunCanvasContactList;
import com.funambol.mailclient.ui.view.FunCanvasViewMessage;
import com.funambol.mailclient.ui.view.FunambolAlert;
import com.funambol.mailclient.ui.view.GetMailSettingsForm;
import com.funambol.mailclient.ui.view.LinkList;
import com.funambol.mailclient.ui.view.LogSettingsForm;
import com.funambol.mailclient.ui.view.LogViewerForm;
import com.funambol.mailclient.ui.view.RoundedDrawer;
import com.funambol.mailclient.ui.view.Drawer;
//import com.funambol.mailclient.ui.view.PhoneNumberList;
import com.funambol.mailclient.ui.view.SettingsList;
import com.funambol.mailclient.ui.view.FunCanvasDraftMessageList;
import com.funambol.mailclient.ui.view.FunCanvasInboxMessageList;
import com.funambol.mailclient.ui.view.FunCanvasMessageList;
import com.funambol.mailclient.ui.view.FunCanvasOutboxMessageList;
import com.funambol.mailclient.ui.view.FunCanvasSentMessageList;
import com.funambol.mailclient.ui.view.FunCanvasRecipientList;
import com.funambol.mailclient.ui.view.FunModalPopup;
import com.funambol.mailclient.loc.Localization;
import com.funambol.mailclient.cm.Contact;
import com.funambol.mailclient.cm.ContactManager;
import com.funambol.mailclient.cm.ContactManagerException;

import com.funambol.mailclient.mm.FolderInfo;
import com.funambol.mailclient.mm.MessageInfo;
import com.funambol.mailclient.mm.MessageManager;
import com.funambol.mailclient.mm.SyncTaskManager;

import com.funambol.mailclient.config.*;

import com.funambol.mail.Address;
import com.funambol.mail.MailException;
import com.funambol.mail.Message;
import com.funambol.mail.MessageFlags;
import com.funambol.mail.Store;
import com.funambol.mail.StoreFactory;
import com.funambol.syncml.spds.SyncException;
import com.funambol.syncml.spds.SourceConfig;
import com.funambol.syncml.protocol.SyncML;
import com.funambol.util.MailDateFormatter;
import com.funambol.util.StringUtil;
import com.funambol.util.ThreadPool;
import com.funambol.util.Log;
import com.funambol.util.ThreadPoolMonitor;
import java.io.IOException;
import java.io.InputStream;

import java.util.Calendar;
import java.util.Date;
import java.util.TimeZone;

import javax.microedition.io.ConnectionNotFoundException;
import javax.microedition.lcdui.Alert;
import javax.microedition.lcdui.AlertType;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Item;
import javax.microedition.midlet.MIDlet;
import javax.microedition.rms.RecordStoreException;

/**
 * It is the Controller class that controls all interactions
 * between screens and the model
 * It maintains a pool of shared UI instances
 */
public class UIController {
    
    private static final String RE=Localization.getMessages().RE_LABEL;
    private static final String FW=Localization.getMessages().FW_LABEL;
    
    public static final int SENT_MESSAGE=0;
    public static final int OUTGOING_MESSAGE=1;
    public static final int INCOMING_MESSAGE=2;
    public static final int COMMAND_TYPE = Command.OK;
    
    public static Funambol midlet;
    public static Display display;
    
    // The command pool
    public static Command backCommand =
            new Command(Localization.getMessages().BACK_COMMAND,
            Command.BACK, 0);
    public static Command selectCommand =
            new Command(Localization.getMessages().SELECT_COMMAND,
            UIController.COMMAND_TYPE, 0);
    public static Command cancelCommand =
            new Command(Localization.getMessages().CANCEL_COMMAND,
            Command.BACK, 10);
    public static Command saveCommand =
            new Command(Localization.getMessages().SAVE_COMMAND,
            UIController.COMMAND_TYPE, 1);
    public static Command inboxCommand =
            new Command(Localization.getMessages().INBOX_COMMAND,
            UIController.COMMAND_TYPE, 1);
    
    // The contact manager
    private static ContactManager contactManager;
    
    // The screen pool
    public static ComposeMessageForm composeMessageScreen;
    
    private static FunCanvasInboxMessageList inboxMessageList;
    private static FunCanvasOutboxMessageList outboxMessageList;
    private static FunCanvasSentMessageList sentMessageList;
    private static FunCanvasDraftMessageList draftMessageList;
    protected static FunCanvasContactList contactList;
    
    // The screen stack
    public static Stack backStack = new Stack();
    
    // Thread pool
    private static ThreadPool threadPool;
    private ThreadPoolMonitor uiThreadMonitor;
    private static final int MAX_NUM_THREADS = 4;
    
    public static MessageManager messageManager;
    public static MailClientConfig mailClientConfig;
    
    // Folder informations
    private static FolderInfo[] folderInfo;
    public static boolean otaStarted;
    
    //public static long startAppProbe;
    
    public static long startupFreeMem;
    
    //private static long heapSize = Runtime.getRuntime().totalMemory();
    
    private static ContactListLoader contactListLoader;
    
    private static MessageWriter messageWriter;
    
    private static SyncTaskManager syncTaskManager;
    
    private static Drawer drawer;
    
    
    
    private UIController() {
    }
    
    public static void initThreadPool() {
        ThreadPoolMonitor uiThreadMonitor = new UIThreadPoolMonitor();
        threadPool=new ThreadPool(uiThreadMonitor, MAX_NUM_THREADS);
    }
    
    public static ThreadPool getThreadPool() {
        return threadPool;
    }
    
    /**
     * enable or disable the sync and reset commands from inbox and contact lists,
     * the send command from compose and the
     * if needed
     *
     * @param enable enable the commands if true, disable the commands if false
     */
    public static void enableSyncCommands( boolean enable ) {
        
        // TODO: add the remove for send log command
        
        if (contactList != null) {
            contactList.enableSyncCommand(enable);
        }
        if (inboxMessageList != null)  {
            getInboxMessageList().enableSyncCommand(enable);
        }
        if (composeMessageScreen!=null) {
            getComposeMessageScreen().enableSendCommand(enable);
        }
    }
    
    
    /**
     * init the inbox: calls initmessagemanager, create the cache and display
     * the inbox with the cached messages, then starts the thread that loads the
     * messages from the rms
     *
     * @return true if the account has been set, false otherwise.
     * The account is considered set if the url is different from
     * Account.DEFAULT_URL and if the username and password are not empty.
     *
     */
    public static boolean initInbox() {
        boolean accountSet = false;
        
        try {
            initMessageManager();
            
            //long startInbox = System.currentTimeMillis();//XXX:time
            
            //long getCache = System.currentTimeMillis();//XXX:time
            
            MessageInfo[] cacheList = Cache.getMsgSnapShot(Cache.INBOX);
            
            //long gotCache = System.currentTimeMillis();//XXX:time
            
            // Log.debug("Got Snapshot");
            
            //  Log.debug("Cache list size = " + cacheList.length);
            
            //create the inbox using the cache
            inboxMessageList = new FunCanvasInboxMessageList(cacheList);
            outboxMessageList = null;
            draftMessageList = null;
            sentMessageList = null;
            
            accountSet = mailClientConfig.getMailAccount().isSet();
            // If the account information has been already set
            if (accountSet) {
                display.setCurrent(inboxMessageList);
                
                // this starts the thread that load messages from rms
                getThreadPool().startThread(inboxMessageList);
            
            } else {
                Log.info("initInbox: go to account settings.");
                
                // Some devices, like the RAZR, don't show two display
                // if the calls to display.setcurrent are too close.
                //sleep(10);
                
                showAlert(Localization.getMessages().CONFIG_NOT_SET,
                        getAccountSettingsForm(inboxMessageList));
            }
            
            /*
            long endInbox = System.currentTimeMillis();//XXX:time
            Log.addProbe("\n_probe_start_cache = " + getCache);//XXX:time
            Log.addProbe("\n_probe_end_cache = " + gotCache);//XXX:time
            Log.addProbe("\n_probe_start_createInbox = " + startInbox);//XXX:time
            Log.addProbe("\n_probe_end_createInbox = " + endInbox);//XXX:time
            Log.addProbe("\n_probe_start_showInbox = " + startAppProbe);//XXX:time
            Log.addProbe("\n_probe_end_showInbox = " + System.currentTimeMillis());
             */
        } catch (Exception ex) {
            Log.error("UIController: error starting sync or getting cache" + ex);
            ex.printStackTrace();
            //if (display.getCurrent() != inboxMessageList) {
            //    Log.error("inboxmessagelist not shown, resetting it");
            //    resetInboxMessageList();
            //    display.setCurrent(getInboxMessageList());
            //}
        }
        
        return accountSet;
    }
    
    /**
     * Suspend the current thread for the given number of seconds
     *
     * @param millis the time to wait (milliseconds)
     */
    public static void sleep(int millis) {
        try {
            Thread.sleep(millis);
        } catch (Exception e) {
            Log.error("sleep interrupted.");
        };
    }
    /**
     * mesure the currently used memory
     *
     * @return current used memory in bytes
     */
    public static long startMemMeasure() {
        //Runtime.getRuntime().gc();
        long totalb4 = Runtime.getRuntime().totalMemory();
        long freeb4 = Runtime.getRuntime().freeMemory();
        long usedb4 = totalb4 - freeb4;
        return usedb4;
    }
    
    
    /**
     * calculate the difference between current used memory and the given used memory,
     *and print a log.error message with the given string
     *
     * @param msg a string that will be used to identify the element
     * that use the memory
     * @param usedb4 the memory used before the element was activated
     *
     */
    public static void calcMem(String msg, final long usedb4) {
        //Runtime.getRuntime().gc();
        long totalAfter = Runtime.getRuntime().totalMemory();
        long freeAfter = Runtime.getRuntime().freeMemory();
        long usedAfter = totalAfter - freeAfter;
        Log.error("Memory used by " + msg + " = " + (usedAfter - usedb4)  );
    }
    
    
    /**
     * init the messagemanager and the store
     */
    public static void initMessageManager() {
        try {
            //Log.debug("MessageManager init");
            messageManager = MessageManager.getInstance();
            initStore(false);
            
        } catch (Exception ex) {
            Log.error("UIController: error init MessageManager " + ex);
            ex.printStackTrace();
        }
    }
    
    /**
     * init the message store
     * @param reset if true the folders will be erased and recreated
     */
    public static void initStore(boolean reset) throws MailException {
        messageManager.initStore(reset);
        if (reset) {
            folderInfo=null;
        }
    }
    
    /**
     *
     * showXXX methods managing the views
     *
     */
    
    /**
     * This method retrieves the screen to go back from the Stack
     */
    public static void showBackScreen() /*throws Exception*/ {
        try {
            Displayable screen = backStack.pop();
            // If the screen is the inbox, cleanup the other screens
            if(screen == inboxMessageList) {
                showInboxScreen();
            } else {
                display.setCurrent(screen);
            }
        } catch (Exception ex) {
            Log.error("ShowBackScreen: " + ex.toString());
            showInboxScreen();
        }
    }
    
    
    /**
     * show the current composeMessageScreen, performing no updates. <br />
     *
     * if you need to update the messagescreen use updateComposeMessageScreen
     * functions
     **/
    public static void showComposeMessageScreen(Displayable backScreen, Item item) {
        //backStack.push(backScreen);
        //display.setCurrent(getComposeMessageScreen());
        if (backScreen!=null) {
            backStack.push(backScreen);
        } else {
            backStack.push(inboxMessageList);
        }
        getComposeMessageScreen().setDefaultTitle();
        //display.setCurrent(getComposeMessageScreen());
        display.setCurrentItem(item);
    }
    
    /**
     * show the compose message screen ready for a new message to be composed
     */
    public static void composeNewMessage(Displayable backScreen) {
        
        // TODO: reset the contact list withoud reloading from rms, we can
        // use the contacts stored in memory!
        
        //threadPool.startThread(getContactListLoader(new Message()));
        //getComposeMessageScreen().get
        getContactListLoader(new Message()).load();
        getContactList().emptySearchString();
        showAddressList(display.getCurrent());
    }
    
    /**
     * update ComposeMessageScreen message.
     * the message remains unchanged
     **/
    public static void updateComposeMessageScreen(Message message) {
        getComposeMessageScreen().setMessage(message);
    }
    
    /**
     * update ComposeMessageScreen toAppend string.
     * the toAppend string remains unchanged
     **/
    public static void updateComposeMessageScreen(String toAppend) {
        getComposeMessageScreen().setToAppend(toAppend);
    }
    
    
    /**
     * update ComposeMessageScreen toAppend string and message
     **/
    public static void updateComposeMessageScreen(Message message, String toAppend) {
        updateComposeMessageScreen(message);
        updateComposeMessageScreen(toAppend);
    }
    
    /**
     * update ComposeMessageScreen message with the message referenced
     * by given messageinfo
     * the toAppend string remains unchanged
     **/
    public static void updateComposeMessageScreen(MessageInfo messageInfo) {
        Message toShow;
        try {
            toShow = messageInfo.getMessage();
            updateComposeMessageScreen(toShow);
        } catch (MailException ex) {
            ex.printStackTrace();
            Log.error("UIController: Unable to get message from messageInfo");
            //TODO: add alert here
        }
    }
    
    /**
     * update ComposeMessageScreen toAppend string and
     *  message (using the message referenced by given messageInfo)
     **/
    public static void updateComposeMessageScreen(MessageInfo messageInfo, String toAppend) {
        updateComposeMessageScreen(messageInfo);
        updateComposeMessageScreen(toAppend);
    }
    
    /**
     * return the composemessageform and create it if needed
     */
    public static ComposeMessageForm getComposeMessageScreen() {
        if (composeMessageScreen==null) {
            composeMessageScreen=new ComposeMessageForm();
        }
        return composeMessageScreen;
    }
    
    /**
     *  show the inbox screen, cleaning the stack and freeing not used objects
     * and memory
     */
    public static void showInboxScreen() {
        backStack.clear();
        outboxMessageList = null;
        sentMessageList = null;
        draftMessageList = null;
        composeMessageScreen = null;
        // contactList = null;
        if(!SyncClient.isBusy()) {
            getInboxMessageList().resetTitle();
        }
        display.setCurrent(getInboxMessageList());
        //#ifdef low_mem
//#         freeMemory();
        //#endif
    }
    
    /** If the freeMemory is low, try to collect some memory */
    public static long freeMemory() {
        
        long mem = Runtime.getRuntime().freeMemory();
        if(mem < 10000) {
            Log.info(">> freeMemory is low: " + Runtime.getRuntime().freeMemory());
            try {
                System.gc();
                Thread.sleep(1);
            } catch (InterruptedException ex) {
                // just ignore it
                Log.error("interrupted exception freeing memory");
                
            }
            mem = Runtime.getRuntime().freeMemory();
        }
        
        return mem;
    }
    
    /**
     * Display the outbox screen.
     */
    public static void showOutboxScreen(Displayable backScreen) {
        backStack.push(backScreen);
        //TODO: use lazy initInbox
        updateOutboxMessageList(null);
        display.setCurrent(getOutboxMessageList());
    }
    /**
     * push current screen in the backstack and display the sent message list
     *
     *@param backscreen the displayable to be pushed in the stack,
     * often the current displayable
     */
    public static void showSentScreen(Displayable backScreen) {
        backStack.push(backScreen);
        updateSentMessageList(null);
        UIController.display.setCurrent(getSentMessageList());
    }
    
    /**
     * push current screen in the backstack and display the draft message list
     *
     *@param backscreen the displayable to be pushed in the stack,
     * often the current displayable
     */
    public static void showDraftScreen(Displayable backScreen) {
        backStack.push(backScreen);
        updateDraftMessageList(null);
        UIController.display.setCurrent(getDraftMessageList());
    }
    
    /**
     *show the given messageInfo's Message
     *
     *@param backScreen the screen to be pushed in the stack
     *@param messageInfo the messageInfo pointing to the message to be shown
     *@param messageType one of UIController.OUTGOING_MESSAGE or
     *   UIController.INCOMING_MESSAGE
     */
    public static void showViewMessageScreen(Displayable backScreen,
            MessageInfo messageInfo) {
        try {
            //Log.info("showViewMessageScreen:" + System.currentTimeMillis());
            showViewMessageScreen(backScreen, messageInfo.getMessage());
        } catch (MailException ex) {
            ex.printStackTrace();
            Log.error("ViewMessage: "+ ex.toString());
            UIController.showErrorAlert(
                    Localization.getMessages().GENERIC_UNABLE_TO_OPEN_MESSAGE);
        }
    }
    
    /**
     * push current screen in the backstack and display the view message screen
     * with the given message.
     *<br>
     *the message read flag will be set to true if needed
     *
     *
     *
     * @param backscreen the displayable to be pushed in the stack,
     * often the current displayable
     * @param message the message to be displayed
     */
    
    public static void showViewMessageScreen(Displayable backScreen, Message message) {
        backStack.push(backScreen);
        //   Log.debug("message flag opened = " + message.getFlags().isSet(MessageFlags.OPENED));
        if (!message.getFlags().isSet(MessageFlags.OPENED)) {
            changeReadFlag(message);
        }
        FunCanvasViewMessage viewMessageScreen = new FunCanvasViewMessage(message);
        FunCanvasMessageList fcml = (FunCanvasMessageList) backScreen;
        viewMessageScreen.setTitle("Message " + (fcml.getActiveElement()+1)
        + " of " + fcml.getNumMessages());
        display.setCurrent(viewMessageScreen);
    }
    
    /**
     * Create the folder list and display it
     * @param backscreen the displayable to be pushed in the stack,
     * often the current displayable
     */
    public static void showFolderList(Displayable displayable) {
        backStack.push(displayable);
        FolderInfo[] folders=messageManager.getRootFolders();
        display.setCurrent(new FolderList(folders));
    }
    
    /**
     * opens a list of links in the given message body
     * @param body the string to be searched for links
     * @param backscreen the displayable to be pushed in the stack,
     * often the current displayable
     *
     */
    public static void showLinkList(Displayable displayable, String body) {
        backStack.push(displayable);
        display.setCurrent(new LinkList(body));
    }
    
    /*
    public static void showPhoneNumberList(Displayable displayable, String body) {
        backStack.push(displayable);
        display.setCurrent(new PhoneNumberList(body));
    }
     */
    /**
     * Methods to show settings screens
     * @param backscreen the displayable to be pushed in the stack,
     * often the current displayable
     *
     */
    public static void showSettings(Displayable displayable) {
        backStack.push(displayable);
        display.setCurrent(new SettingsList());
    }
    
    
    /**
     *show account settings form
     * @param backscreen the displayable to be pushed in the stack,
     * often the current displayable
     *
     */
    public static void showAccountSettings(Displayable displayable) {
        display.setCurrent(getAccountSettingsForm(displayable));
    }
    
    /**
     * @return the account settings form
     * @param backscreen the displayable to be pushed in the stack,
     * often the current displayable
     */
    public static Displayable getAccountSettingsForm(Displayable displayable) {
        backStack.push(displayable);
        return new AccountSettingsForm();
    }
    /**
     * show the mail sync settings form
     * @param backscreen the displayable to be pushed in the stack,
     * often the current displayable
     */
    public static void showMailSyncSettings(Displayable displayable) {
        backStack.push(displayable);
        display.setCurrent(new GetMailSettingsForm());
    }
    
    
    /**
     * show the log settings form
     * @param backscreen the displayable to be pushed in the stack,
     * often the current displayable
     */
    public static void showLogSettings(Displayable displayable) {
        backStack.push(displayable);
        display.setCurrent(new LogSettingsForm());
    }
    
    /**
     * show the log view form
     * @param backscreen the displayable to be pushed in the stack,
     * often the current displayable
     */
    
    public static void showLogViewer(Displayable displayable) {
        backStack.push(displayable);
        display.setCurrent(new LogViewerForm());
    }
    
    /**
     * show the recipient list form
     * @param backscreen the displayable to be pushed in the stack,
     * often the current displayable
     */
    
    public static void showRecipientsList(Displayable displayable, Message message) {
        backStack.push(displayable);
        display.setCurrent(new FunCanvasRecipientList(message));
    }
    
    /**
     * show the contact list
     * @param backscreen the screen to be put in the stack for future back commands
     */
    public static void showAddressList(Displayable backScreen) {
        backStack.push(backScreen);
        getContactList().rewindList();
        display.setCurrent(getContactList());
    }
    
    
    public static void showAddContactScreen(Displayable displayable,
            FunCanvasContactList addressList) {
        backStack.push(displayable);
        display.setCurrent(new AddContactForm(addressList));
    }
    
    public static void showAddContactScreen(Displayable displayable) {
        showAddContactScreen(displayable,(FunCanvasContactList) null);
    }
    
    
    public static void showAddContactScreen(Displayable displayable, Contact contact) {
        backStack.push(displayable);
        UIController.display.setCurrent(new AddContactForm(contact,getContactList(), true));
    }
    
    /**
     * displays a edit contact screen and update the given FunAddressList after
     * the edit.
     *
     *
     * @param displayable: the displayable to be pushed in the stack
     * contact: the contact to be edited
     * contactList: the adddresslist to be updated (can be null if not needed)
     */
    public static Displayable showEditContactScreen(Displayable displayable,
            Contact contact, FunCanvasContactList contactList, boolean isNew) {
        backStack.push(displayable);
        Displayable contactForm=new AddContactForm(contact, contactList, isNew);
        UIController.display.setCurrent(contactForm);
        return contactForm;
    }
    
    /**
     * Show an info alert with autodismiss
     */
    public static void showAlert(String message) {
        showAlert(message, AlertType.INFO, true);
    }
    
    /**
     * Show an info alert with autodismiss and a displayable
     * to open after it's been dismissed.
     */
    public static void showAlert(String message, Displayable displayable) {
        showAlert(message, AlertType.INFO, true, displayable);
        
    }
    
    /**
     * Show an error alert without autodismiss
     */
    public static void showErrorAlert(String message) {
        showAlert(message, AlertType.ERROR, false);
    }
    
    /**
     * Show an info alert without autodismiss and a displayable
     * to open after it's been dismissed.
     */
    public static void showErrorAlert(String message, Displayable displayable) {
        showAlert(message, AlertType.ERROR, false, displayable);
    }
    
    /**
     * Show an alert with given message, alert type and autodismiss.
     * This is the generic method called by the others.
     */
    private static void showAlert(String message, AlertType alertType, boolean autoDismiss) {
        Alert messageAlert = getFunambolAlert(message, alertType, autoDismiss);
        display.setCurrent(messageAlert);
    }
    
    /**
     * Show an alert with given message, alert type, autodismiss and next displayable
     */
    public static void showAlert(String message, AlertType alertType,
            boolean autoDismiss, Displayable displayable) {
        Alert messageAlert = getFunambolAlert(message, alertType, autoDismiss);
        display.setCurrent(messageAlert, displayable);
    }
    
    public static void showAbout(Displayable displayable) {
        backStack.push(displayable);
        //display.setCurrent(new AboutForm());
        display.setCurrent(new AboutScreen());
    }
    
    
    /**
     * Utilites methods managing the Model
     */
    
    
    public static void replyMessage(Message message) {
        Message replyMessage=new Message();
        //TODO: exception handling
        try {
            replyMessage.setTo(message.getReplyTo());
            replyMessage.setSubject(RE + message.getSubject());
        } catch (MailException ex) {
            ex.printStackTrace();
            Log.error("unable to set content into reply " +
                    "message, error getting 'from' field from message");
        }
        delayedFlag(message, MessageFlags.ANSWERED);
        updateComposeMessageScreen(replyMessage,getReplyText(message));
        getContactList().emptySearchString();
        showComposeMessageScreen(display.getCurrent(),getComposeMessageScreen().body);
    }
    
    
    
    /**
     * set the message as the original message in composemessagescreen, the next
     * time the send / sendlater commandaction is performed the given flag will
     * be set to true
     *
     * @param message the message to be set as the original message in
     * composemessageform
     *
     * @param flag the flag (one of MessageFlags constants) to be set to true
     */
    private static void delayedFlag(final Message message, final int flag) {
        //Log.debug("UIController: trying to set read message flag");
        //setRepliedFlag(message);
        MessageFlags flags = new MessageFlags();
        flags.setFlags(message.getFlags().getFlags());
        if (!flags.isSet(flag)) {
            flags.setFlag(flag, true);
            getComposeMessageScreen().setFlagsToSet(flags);
            try {
                getComposeMessageScreen().setOriginalMessage(
                        new MessageInfo(message, new FolderInfo(
                        message.getParent().getName(), StoreFactory.getStore())));
            } catch (MailException ex) {
                Log.error("error forwarding message while creating messageinfo " +
                        "from original message");
                ex.printStackTrace();
            }
        }
    }
    
    
    /**
     * create a new forward message and show the compose message screen
     *
     * @param message the message to be forwarded (body content will be appended
     * at the bottom of the email)
     */
    public static void forwardMessage(Message message) {
        
        Message forwardMessage=new Message();
        
        forwardMessage.setSubject(FW + message.getSubject());
        /*try {
            messageManager.updateMessageFlag(message, MessageFlags.FORWARDED, true);
        } catch (MailException ex) {
            Log.error("forwardMessage: "+ex.toString());
            ex.printStackTrace();
        }*/
        
        delayedFlag(message, MessageFlags.FORWARDED);
        UIController.updateComposeMessageScreen(forwardMessage,getForwardText(message));
        getContactList().emptySearchString();
        showComposeMessageScreen(display.getCurrent(), getComposeMessageScreen().recipientStringItem);
        
    }
    
    
    
    /**
     * update the message list that contain the given message
     *
     * @param message the message
     */
    private static void updateMessageList(Message message) {
        if (message.getParent()==null) {
            //updateAllMessageLists(message);
            Log.error("Message without parent");
            return;
        } else {
            if (message.getParent().getName().equals(Store.INBOX)) {
                //Log.debug("UIController: message from inbox, updating inbox");
                updateInboxMessageList(message);
            } else if (message.getParent().getName().equals(Store.OUTBOX)) {
                //Log.debug("UIController: message from outbox, updating outbox");
                updateOutboxMessageList(message);
            } else if (message.getParent().getName().equals(Store.SENT)) {
                //Log.debug("UIController: message from sent, updating sent");
                updateSentMessageList(message);
            }  else if (message.getParent().getName().equals(Store.DRAFTS)) {
                //Log.debug("UIController: message from draft, updating draft");
                updateDraftMessageList(message);
            } else {
                Log.error("Unkown folder");
            }
        }
        
    }
    
    /*
    private static void updateAllMessageLists(Object o) {
        updateInboxMessageList(o);
        updateOutboxMessageList(o);
        updateDraftMessageList(o);
        updateSentMessageList(o);
    }*/
    
    public static void updateMessageList(Object o, int theFolder) {
        FolderInfo folderInfo=getFolderInfo(theFolder);
        FunCanvasMessageList messageList;
        switch ( theFolder ) {
            case MessageManager.INBOX:
                messageList = getInboxMessageList();
                break;
            case MessageManager.OUTBOX:
                messageList = getOutboxMessageList();
                break;
            case MessageManager.DRAFTS:
                messageList = getDraftMessageList();
                break;
            case MessageManager.SENT:
                messageList = getSentMessageList();
                break;
            default:
                messageList = getInboxMessageList();
        }
        if (o == null) {
            messageList.setTitle(messageList.getDefaultTitle());
            messageList.setMessageInfoList(getSortedMessageInfo(folderInfo));
        } else if (o instanceof Message)  {
            messageList.setMessageInfoList(getSortedMessageInfo(folderInfo));
        } else if (o instanceof String) {
            if (o.toString().length() == 0 ) {
                messageList.setTitle(messageList.getDefaultTitle());
            } else {
                messageList.setTitle(o.toString());
            }
        }
        
    }
    
    
    public static void updateDraftMessageList(Object o) {
        updateMessageList(o,MessageManager.DRAFTS);
    }
    
    public static void updateSentMessageList(Object o) {
        updateMessageList(o,MessageManager.SENT);
    }
    
    public static void updateInboxMessageList(Object o) {
        updateMessageList(o,MessageManager.INBOX);
    }
    
    public static void updateOutboxMessageList(Object o) {
        updateMessageList(o,MessageManager.OUTBOX);
    }
//    private static void debug(String s) {
//        Log.debug("UIController:"+ s);
//    }
    
//    private static void error(String s) {
//        Log.error("UIController:"+ s);
//    }
    
    /**
     * open a url.
     *
     * @param url the url to be opened in the default browser (device dependent)
     */
    public static void openUrl(String url) {
        Log.info("Opening url: [" + url + "]");
        try {
            // In Nokia S60 device the platformRequest
            // doesn't open link without "http://"
            if (url.startsWith("www"))
                url = "http://"+url;
            midlet.platformRequest( url);
        } catch (ConnectionNotFoundException ex) {
            ex.printStackTrace();
            Log.error("openUrl: " + ex.toString());
            showErrorAlert("Error opening URL.");
            
        } catch (Exception e) {
            e.printStackTrace();
            Log.error("openUrl: " + e.toString());
        }
        
    }
    
    /**
     * save tha given contact
     *
     * @param contact the contact to be saved
     * @param isNew true if contact is new, false if editing existent contact
     * @trows AlreadyExistentContactException if contact already exists
     */
    public static void saveContact(Contact contact, boolean isNew)
    throws ContactManagerException, AlreadyExistentContactException  {
        if (isNew) {
            contactManager.addContact(contact, true);
        } else {
            contactManager.updateContact(contact);
        }
        showBackScreen();
    }
    
    /** getSortedMessageInfo is synchronized to avoid potential issues
     * on concurrent access to the INbox elements in the storage
     * @return a sorted messageinfo with items from the given folderinfo
     */
    public synchronized static MessageInfo[] getSortedMessageInfo(FolderInfo folder) {
        MessageInfo[] list = null;//new MessageInfo[0];
        try {
            list = folder.getMsgList();
            if (list.length>0) {
                return bubbleSortPerDate(list);
            } else {
                return list;
            }
        } catch (Exception ex) {
            Log.error("Error occurred sorting message list: " + ex.getMessage());
            ex.printStackTrace();
            
            return list;
        }
        
    }
    
    
    /**
     * A sort algorithm to order the messages in the view Inbox as per sent date
     */
    public static MessageInfo[] bubbleSortPerDate(MessageInfo[] list) {
        int n = list.length;
        for (int pass = 1; pass < n; pass++) {  // count how many times
            // This next loop becomes shorter and shorter
            for (int i = 0; i < n-pass; i++) {
                if (list[i].received.getTime() < list[i+1].received.getTime()) {
                    // exchange elements
                    MessageInfo temp = list[i];
                    list[i] = list[i+1];
                    list[i+1] = temp;
                }
            }
        }
        
        return list;
    }
    
    /**
     * Nullify addressList reference (current implementation does NOTHING.
     *
     */
    
    //TODO: remove this ASAP if it's confirmed as useless
    
    public static void destroyAddressList() {
        //    contactList=null;
        
        //   getContactList().resetContactList(new Message());
    }
    
    
    /**
     * reset the account deleting the store and startng a new
     * refresh_from_server sync
     */
    public static void resetAccount() {
        try {
            // For RAZR bug: wait before showing a new displayable
            //UIController.sleep(10);
            //UIController.showAlert(Localization.getMessages().RESETTING_ACCOUNT_MSG);
            
            //delete message store
            messageManager.initStore(true);
            //delete contact list and related PIM Item Queue
            if (contactManager==null) {
                contactManager=
                        ContactManagerFactory.getContactManager(ContactManager.RMS_TYPE);
            }
            // reset the inbox view
            resetInboxMessageList();
            showInboxScreen();
            
            // empty the contact list (to free memory)
            contactManager.resetContactList();
            if (contactList != null) {
                contactList.resetContactList(new Message());
            }
            //initialize new account's messages and contacts
            sync(SyncML.ALERT_CODE_REFRESH_FROM_SERVER, true);
            getInboxMessageList().setMinimumDisplayedMessageCount(0);
        } catch (RecordStoreException rse) {
            Log.error("resetAccount: " + rse.toString());
            // TODO: ALERT ??
            showInboxScreen();
        } catch (MailException me) {
            Log.error("resetAccount: " + me.toString());
            // TODO: ALERT ??
            showInboxScreen();
        }
    }
    
    /**
     * show the inbox and requests a refresh from server sync
     */
    public static void resetInboxFromServer() throws RecordStoreException, MailException {
       
        showInboxScreen();
        Log.debug("starting refresh from server...");
        sync(SyncML.ALERT_CODE_REFRESH_FROM_SERVER, false);
    }
    
    
    /**
     * reinit the contacts from server
     */
    public static void resetContacts() {
        try {
            contactManager.resetContactList();
            if (contactList != null) {
                contactList.resetContactList(new Message());
            }
            SyncTaskManager stm = getSyncTaskManager();
            stm.setSyncMessages(false);
            stm.setResetContacts(true);
            stm.setSyncMode(SyncML.ALERT_CODE_REFRESH_FROM_SERVER);
            if (!stm.addObserver(new MessageListUpdater())) {
                Log.error("SyncTaskManager reached max number of observers! ");
            }
            
            getThreadPool().startThread(stm);
            
            
        } catch (SyncException ex) {
            ex.printStackTrace();
            Log.error("SyncException in resetcontacts " + ex.toString());
        }
        // sync(true);
    }
    
    public static void updateLogLevel(int logLevel) {
        Log.setLogLevel(logLevel);
        mailClientConfig.setLogLevel(logLevel);
        saveNewConfig(mailClientConfig);
    }
    
    public static FunCanvasInboxMessageList getInboxMessageList() {
        if (inboxMessageList==null) {
            MessageInfo[] list =
                    getSortedMessageInfo(getFolderInfo(MessageManager.INBOX));
            inboxMessageList=new FunCanvasInboxMessageList( list );
        }
        return inboxMessageList;
    }
    
    private static FunCanvasOutboxMessageList getOutboxMessageList() {
        if (outboxMessageList==null) {
            outboxMessageList=new FunCanvasOutboxMessageList(
                    getSortedMessageInfo(getFolderInfo(messageManager.OUTBOX)));
        }
        return outboxMessageList;
    }
    
    private static FunCanvasSentMessageList getSentMessageList() {
        if (sentMessageList==null) {
            sentMessageList=new FunCanvasSentMessageList(
                    getSortedMessageInfo(getFolderInfo(MessageManager.SENT)));
        }
        return sentMessageList;
    }
    
    private static FunCanvasDraftMessageList getDraftMessageList() {
        if (draftMessageList==null) {
            draftMessageList=new FunCanvasDraftMessageList(
                    getSortedMessageInfo(getFolderInfo(MessageManager.DRAFTS)));
        }
        return draftMessageList;
    }
    
    
    /**
     * change the followup flag to the given messageinfo and start a separate
     * thread for saving the change to the rms
     */
    public static void changeFollowUpFlag(final MessageInfo messageInfo) {
        
        boolean isFlagged = messageInfo.flags.isSet(MessageFlags.FLAGGED);
        messageInfo.flags.setFlag(MessageFlags.FLAGGED, !isFlagged);
        
        UIController.getThreadPool().startThread(
                new Thread() {
            public void run() {
                try {
                    changeFlag(messageInfo.getMessage(), MessageFlags.FLAGGED);
                } catch (MailException ex) {
                    Log.error("MailException changing followup flag");
                    ex.printStackTrace();
                }
            }
        }
        );
        
    }
    
    public static void changeFollowUpFlag(Message message) {
        changeFlag(message, MessageFlags.FLAGGED);
    }
    
   /* public static void changeReadFlag(final MessageInfo messageInfo) {
        boolean isRead = messageInfo.flags.isSet(MessageFlags.OPENED);
        messageInfo.flags.setFlag(MessageFlags.OPENED, !isRead);
    
        UIController.getThreadPool().startThread(
                new Thread() {
            public void run() {
                try {
                    changeFlag(messageInfo.getMessage(), MessageFlags.OPENED);
                } catch (MailException ex) {
                    ex.printStackTrace();
                }
            }
        }
        );
    }*/
    
    public static void changeReadFlag(Message message) {
        Log.debug("changing read flag");
        changeFlag(message, MessageFlags.OPENED);
    }
    
    
    
    /**
     * change the flag on the fly and start a thread to save the message
     * on rms.
     */
    private static void changeFlag(Message message, int flag) {
        boolean isFlagged=message.getFlags().isSet(flag);
        //message.getFlags().setFlag(flag, isFlagged);
        message.getFlags().setFlag(flag,!isFlagged);
        threadedWriteMessageToFolder(message);
        
        if ( message.getParent().getName().equals(Store.INBOX )) {
            //Log.debug("message is in inbox");
            try {
                getInboxMessageList().addMessage(new MessageInfo(
                        message, new FolderInfo(
                        message.getParent().getName(), StoreFactory.getStore()
                        )));
            } catch (MailException ex) {
                ex.printStackTrace();
                Log.error("UIController changeFlag() MailException  changing " +
                        "flag for message " + message.getSubject());
            }
        } else {
            //Log.debug("parent.fullname = " + message.getParent().getFullName());
            //Log.debug("parent.name = " + message.getParent().getName());
            updateMessageList(message);
        }
        
        //message.getFlags().setFlag(flag, !isFlagged);
        //updateMessageList(message);
    }
    
    
    /**
     * save a message to its parent folder and call the messageManager to add
     * the message to the queue
     */
    private static void threadedWriteMessageToFolder(final Message message) {
        
        
        
        // new Thread(new MessageWriter(message)).start();
        
        UIController.threadPool.startThread(getMessageWriter(message));
        
        
        
        /* UIController.threadPool.startThread(new Thread() {
            public void run() {
                Log.debug("messaggio " + message.getSubject());
                //Flags changes notification for eventual sync process
                /*messageManager.updateMessageFlags(message, message.getFlags());
                try {
                   message.getParent().appendMessage(message);
         
                } catch (StoreException ex) {
                    Log.error("threadedWriteMessageToFolder: ", ex.toString() );
                    ex.printStackTrace();
                } catch (MailException ex) {
                    Log.error("threadedWriteMessageToFolder: ", ex.toString() );
                    ex.printStackTrace();
                }
        }
        }
        );*/
    }
    
    /**
     * returns the corresponging folderinfo, folder should be one of
     *
     * MessageManager.INBOX
     * MessageManager.OUTBOX
     * MessageManager.DRAFT
     * MessageManager.SENT
     */
    public static FolderInfo getFolderInfo(int folder) {
        
        if (folderInfo==null)
            folderInfo =  messageManager.getRootFolders();
        
        return folderInfo[folder];
    }
    
    public static void sendMessage(Message message) {
        UIController.showInboxScreen();
        
        /*
         * this is to avoid that a message edited and sent from within the
         * Drafts once sent remains in the Drafts while it is in the Outbox
         */
        if (message.getParent() != null) {
            if (Store.DRAFTS.equals(message.getParent().getName())) {
                try {
                    message.getParent().deleteMessage(message);
                } catch (MailException ex) {
                    Log.error("sendMessage: "+ex);
                    ex.printStackTrace();
                }
            }
        }
        
        try {
            message.setSentDate(new Date());
            message.setReceivedDate(new Date());
            getFolderInfo(messageManager.OUTBOX).getFolder().appendMessage(message);
            
            
            //sync(false);
            
        } catch (Exception ex) {
            Log.error("Error moving message to Outbox");
            ex.printStackTrace();
            showErrorAlert(Localization.getMessages().SAVE_OUTBOX_ERROR);
        } finally {
            //    destroyAddressList();
        }
        
        sync(false);
        
    }
    
    /**
     * Start the sync of the messages, and optionally of the contacts
     * if the parameter syncContacts is true. The sync is done using the
     * default sync type defined for the source.
     */
    public static void sync(boolean syncContacts) {
        sync(-1, syncContacts);
    }
    
    /**
     * Start the SyncTaskManager to send/receive email and, if
     * syncContacts is true, also to update the address book.
     * NOTE: if the contact sync is disabled in the config,
     *       the contact sync is not started, regardless of the value of
     *       the parameter syncContacts.
     *
     * @param syncMode the mode of the sync, as defined in SyncML class.
     *                 If the value -1 is passed, the default
     *                 value defined for the source will be used.
     * @param syncContacts true if the SyncTaskManager should sync
     *                     the contacts too.
     */
    public static void sync(int syncMode, boolean syncContacts) {
        
        //Log.debug("Sync starting...");
        try {
            SyncTaskManager stm = getSyncTaskManager();
            if(syncMode != -1) {
                stm.setSyncMode(syncMode);
            }
            stm.setResetContacts(syncContacts);
            stm.setSyncMessages(true);
            //     Log.info(">>stm prepared: "+Runtime.getRuntime().freeMemory());
            
            stm.addObserver(new MessageListUpdater());
            getThreadPool().startThread(stm);
            //      Log.info(">>stm started: "+Runtime.getRuntime().freeMemory());
            
        } catch (Exception e) {
            e.printStackTrace();
            Log.error("UIController.sync: " + e.toString());
        }
        //Log.debug("Thread for sync started...");
        
    }
    
    public static void sendLaterMessage(Message message) {
        try {
            /*
             * this is to avoid that a message in the Outbox once edited and
             * saved (i.e. stored in the Drafts) remains in the Outbox while
             * it is in the Drafts
             */
            if (message.getParent() != null) {
                if ("/Drafts".equals(message.getParent().getFullName()))
                    getFolderInfo(messageManager.DRAFTS).getFolder().deleteMessage(message);
            }
            message.setSentDate(new Date());
            message.setReceivedDate(new Date());
            getFolderInfo(messageManager.OUTBOX).getFolder().appendMessage(message);
            //Log.debug("Message moved to outbox: "+ message.getParent().getName());
        } catch (Exception ex) {
            Log.error("SendLaterMessage: " + ex.toString());
            ex.printStackTrace();
        }
        //updateOutboxMessageList(message);
        //updateDraftMessageList(message);
        //updateInboxMessageList(null);   //XXX why this???
        
        showAlert(
                Localization.getMessages().COMPOSE_MESSAGE_FORM_SAVED_TO_OUTBOX,
                getInboxMessageList());
        
        // destroyAddressList();
    }
    
    public static void saveMessage(Message message) {
        try {
            
            /*
             * this is to avoid that a message in the Drafts once edited and
             * saved to be sent later (i.e. stored in the Outbox) remains in the
             * Drafts while it is in the Outbox
             */
            if (message.getParent() != null)
                if ("/Outbox".equals(message.getParent().getFullName()))
                    getFolderInfo(messageManager.OUTBOX).getFolder().deleteMessage(message);
            
            getFolderInfo(messageManager.DRAFTS).getFolder().appendMessage(message);
            messageManager.updateMessageFlag(message, MessageFlags.DRAFT, true);
            //message.getFlags().setFlag(MessageFlags.DRAFT, true);
            //updateDraftMessageList(message);
            //updateInboxMessageList(null);
            showAlert(
                    Localization.getMessages().COMPOSE_MESSAGE_FORM_SAVED_TO_DRAFT,
                    getInboxMessageList());
            //destroyAddressList();
            //Log.debug("message saved to draft");
        } catch (Exception ex) {
            Log.error("UIC.savemessage: " + ex.toString() );
            showErrorAlert(
                    Localization.getMessages().UNABLE_TO_SAVE_MESSAGE_TO_DRAFT,
                    getInboxMessageList());
            
            //  destroyAddressList();
        }
    }
    
    
    private static String getMyAddress() {
        mailClientConfig = getConfig();
        return (mailClientConfig.getMailAccount().getAddress());
    }
    
    /**
     * build the reply message of a given message.
     *
     */
//TODO: check the cc field, it seems not to work.
    public static void replyAll(Message message) {
        
        Message m=new Message();
        try {
            m.setCc(message.getCc());
            m.setTo(message.getReplyTo());
            Address[] to=message.getTo();
            for (int i=0; i<to.length; i++) {
                if (!StringUtil.equalsIgnoreCase(to[i].getEmail(),getMyAddress())) {
                    to[i].setType(Address.CC);
                    m.addRecipient(to[i]);
                }
            }
            
            m.setSubject(RE+ message.getSubject());
            //setRepliedFlag(message);
            delayedFlag(message, MessageFlags.ANSWERED);
            updateComposeMessageScreen(m,getReplyText(message));
            getContactList().emptySearchString();
            showComposeMessageScreen(display.getCurrent(),getComposeMessageScreen().body);
            
        } catch (MailException ex) {
            Log.error("Error in reply all message function: " + ex.getMessage());
            ex.printStackTrace();
        }
        
        
        
        
    }
    
    /**
     * Get the complete message (up to the maxObjSize).
     */
    public static Message getFullMessage(Message message) {
        
        Message msg = message;
        
        Log.info("Get Full Message");
        try {
            msg=messageManager.getFullMessage(
                    message.getParent().getName(),
                    message.getMessageId());
        } catch (ConfigException e) {
            Log.error("getFullMessage: " + e.toString());
            e.printStackTrace();
        } catch (MailException e) {
            Log.error("getFullMessage: " + e.toString());
            e.printStackTrace();
        }
        return msg;
    }
    
    private static String getReplyText(Message message) {
        StringBuffer sb = new StringBuffer();
        sb.append("\n\n");
        
        try {
            //TODO: use right timiezone offset here
            Date dateObj = message.getSentDate();
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(dateObj);
            calendar.setTimeZone(TimeZone.getDefault());
            
            // this is only for visualization purposes
            //TODO: check if the variables in formatLocalTime are really used
            String date = MailDateFormatter.formatLocalTime(calendar.getTime());
            
            sb.append(Localization.getMessages().STRING_ON).
                    append(" ").append(date).append(" ").
                    append((message.getFrom()==null)?"":message.getFrom().getVisibleName()).
                    append(" ").append(Localization.getMessages().REPLY_WROTE).
                    append("\n").append(message.getTextContent());
            
        } catch (MailException ex) {
            ex.printStackTrace();
            Log.error("getReplyText() mailexception ");
        }
        
        //Log.debug("replty text: \n----------------------\n" + sb.toString() + "\n-------------------------");
        return sb.toString();
        
    }
    
    private static FunambolAlert getFunambolAlert(String message,
            AlertType alertType, boolean autoDismiss) {
        
        /*FunambolAlert messageAlert =new FunambolAlert(
                Localization.getMessages("FunambolAlertTitle"),
                message,
                alertType);*/
        FunambolAlert messageAlert =new FunambolAlert(
                "Funambol",
                message,
                alertType);
        
        if (autoDismiss)
            messageAlert.setTimeout(FunambolAlert.DEFAULT_TIMEOUT);
        else
            messageAlert.setTimeout(Alert.FOREVER);
        
        return messageAlert;
        
    }
    
    public static Address getMyFrom() {
        try {
            if (StringUtil.isNullOrEmpty(getMyAddress())) {
                return null;
            } else {
                return new Address(
                        Address.FROM,
                        mailClientConfig.getMailAccount().getName(),
                        getMyAddress());
            }
        } catch (MailException ex) {
            ex.printStackTrace();
            UIController.showAccountSettings(display.getCurrent());
            UIController.showErrorAlert(
                    Localization.getMessages().UNABLE_TO_FIND_PERSONAL_ADDRESS
                    );
            Log.error("uicontroller getMyFrom() error " +
                    "getting from field, returning null");
            return null;
        }
    }
    
    /**
     * Returns the current MailClientConfig, or the default one
     * in case of errors.
     */
    public static MailClientConfig getConfig() {
        try {
            return ConfigManager.getConfig();
        } catch (ConfigException e) {
            // TODO shall we alert the user?
            Log.error("Error retrieving configuration: " + e.getMessage());
            MailClientConfig mcc = new MailClientConfig();
            try {
                ConfigManager.saveConfig(mcc);
            } catch (Exception ex) {
                Log.error("!!!Can't save default config!!! " + ex.toString());
            }
            return mailClientConfig;
        }
    }
    
    /**
     * Save the current configuration to the persistent storage.
     */
    public static void saveConfig() {
        try {
            ConfigManager.saveConfig();
        } catch (Exception e) {
            // TODO: shall we alert the user?
            Log.error("Error saving configuration: " + e.getMessage());
        }
    }
    
    /**
     * Save new configuration to persistent memory.
     * If username field is changed a new account is initialized and
     * slow sync are performed both for contact list and Emails
     * @param mailClientConfig is the new configuration to be saved
     */
    public static void saveNewConfig(MailClientConfig mailClientConfig) {
        try {
            ConfigManager.saveConfig(mailClientConfig);
        } catch (Exception e) {
            Log.error("Unable to save new configuration: " + e.getMessage());
            e.printStackTrace();
        }
    }
    
    /**
     * Loads the source config from the store, if present.
     *
     * @param name the name of the source to load.
     * @return the config, or null in case of error.
     */
    public static SourceConfig getSourceConfig(String name) {
        SourceConfig ret = new SourceConfig();
        try {
            ConfigManager.load("source." + name, ret);
            return ret;
            
        } catch (Exception e) {
            Log.error("getSourceConfig: "+ e.toString());
            return null;
        }
    }
    
    /**
     * Save the given source config.
     *
     * @param sc the SourceConfig to save.
     *
     * @throws ConfigException if the config is not present
     * @throws IOException if an error occurred during the deserialization
     */
    public static void saveSourceConfig(SourceConfig sc) {
        try {
            // The source config is stored in a record named "source.<sourcename>"
            ConfigManager.save("source." + sc.getName(), sc);
            Log.info("[UIController] config for source "+sc.getName()+" saved.");
            
        } catch (Exception e) {
            Log.error("saveSourceConfig: "+ e.toString());
            // TODO: shall we put an alert here?
        }
    }
    
    /** Enable or disable contact synchronization. */
    public static void enableContactSync(boolean enable) {
        SourceConfig sc = getSourceConfig(SourceConfig.CONTACT);
        if(sc != null) {
            if (enable) {
                sc.setSyncMode(SyncML.ALERT_CODE_FAST);
            } else {
                sc.setSyncMode(SyncML.ALERT_CODE_NONE);
            }
            saveSourceConfig(sc);
        }
    }
    
    public static void playSound() {
        UIController.display.vibrate(1000);
        InputStream is =
                midlet.getClass().getResourceAsStream("/res/newmessage.midi");
        
        MessageMediaManager mm = new MessageMediaManager(is);
        mm.playSound();
    }
    
    public static void deleteMessage(Message toDelete)  {
        try {
            messageManager.deleteMessage(toDelete);
            if (UiUtils.isIncoming(toDelete)) {
                getInboxMessageList().deleteMessage(toDelete.getMessageId());
            } else {
                updateMessageList(toDelete);
            }
        } catch (Exception ex) {
            ex.printStackTrace();
            Log.error("uicontroller deleteMessage() error deleting" +
                    " message " + toDelete.getSubject());
            showErrorAlert(
                    Localization.getMessages().
                    getGENERIC_UNABLE_TO_DELETE_MESSAGE(toDelete.getSubject()));
        }
        
    }
    
    public static void deleteMessage(MessageInfo msgInfo) {
        deleteMessage(msgInfo.getMessage());
    }
    
    public static String getVersion() {
        return ConfigManager.getClientVersion();
    }
    
    public static void emptyFolder(int folderId) {
        try {
            messageManager.emptyFolder(folderId);
            
        } catch (MailException ex) {
            ex.printStackTrace();
            Log.error("unable to recreate folder " + folderId );
            //TODO: add alert
        }
    }
    
    
    public static void showAdvancedContactScreen(Contact contact) {
        backStack.push(display.getCurrent());
        UIController.display.setCurrent(new AdvancedContactForm(contact));
    }
    
    /**
     * delete a contact
     **/
    public static void deleteContact(Contact contact) {
        try {
            UIController.contactManager.removeContact(contact.getContactId());
            UIController.showAlert(Localization.getMessages().DELETING_CONTACT);
        } catch (ContactManagerException ex) {
            UIController.showErrorAlert(
                    Localization.getMessages().UNABLE_TO_DELETE_CONTACT);
        }
    }
    
    public static void synchContacts() throws SyncException {
        if (contactManager.getContactCount()>=contactManager.getDefaultContactNumber()) {
            Log.info("[UIController] Addressbook is full. Remove some contact from server and try again.");
            UIController.showAlert(Localization.getMessages().AB_LIMIT_ERROR_MSG);
            return;
        }
        SourceConfig sc = getSourceConfig(SourceConfig.CONTACT);
        // Check if the contacty sync is not enabled.
        if(sc != null && sc.getSyncMode() == SyncML.ALERT_CODE_NONE) {
            Log.info("Contact sync disabled.");
        } else {
            
            try {
                enableSyncCommands(false);
                contactManager.sync();
            } catch (ConfigException e) {
                Log.error("ConfigException: " + e.getMessage());
                e.printStackTrace();
            } catch (ContactManagerException e) {
                Log.error("ContactManagerException: " + e.getMessage());
                e.printStackTrace();
            } finally {
                enableSyncCommands(true);
            }
        }
    }
    
    private static String getForwardText(Message message) {
        return getReplyText(message);
    }
    
    
    public  static void createContactManager() {
        try {
            Log.info("Creating Contact Store.");
            contactManager = ContactManagerFactory.getContactManager(
                    ContactManager.RMS_TYPE);
        } catch (RecordStoreException ex) {
            Log.error("createContactManager: " + ex.toString() );
            ex.printStackTrace();
        }
    }
    
    /*
    private static void enlistNewMessage(FunMessageList messageList,
            Message message, FolderInfo folderInfo) {
        try {
            messageList.appendMessageItem(new MessageInfo(message, folderInfo));
        } catch (MailException ex) {
            ex.printStackTrace();
        }
    }
     */
    public static void resetInboxTitle() {
        getInboxMessageList().resetTitle();
    }
    
    /**
     * resort inbox messageinfo list
     */
    public static void resortInboxMessages() {
        getInboxMessageList().setMessageInfoList(
                bubbleSortPerDate(getInboxMessageList().getMessageInfoList()));
        
        //getInboxMessageList().setMessageInfoList(getInboxMessageList().get)
    }
    
    /**
     * reload the inbox messages from rms and reset the inbox title to the default title
     */
    public static void resetInboxMessageList() {
        
        inboxMessageList=getInboxMessageList();
        inboxMessageList.setTitle(inboxMessageList.getDefaultTitle());
        reloadInboxMessageListMessages();
    }
    
    /**
     * reload the inbox message list messages
     * 
     */
    public static void reloadInboxMessageListMessages() {
        FolderInfo folderInfo=getFolderInfo(MessageManager.INBOX);
        getInboxMessageList().setMessageInfoList(getSortedMessageInfo(folderInfo));
    }
    
  
    
    public static void showModalPopup(FunModalPopup popup, Displayable displayable) {
        backStack.push(displayable);
        display.setCurrent(popup);
    }
    
    /**
     * @return the contactmanager
     */
    public static ContactManager getContactManager(){
        if (null == contactManager ){
            createContactManager();
        }
        
        return contactManager;
    }
    
    /**
     * shows the given displayable
     * @param toShow the displayable to show
     * @param backscreen the displayable to push in the stack.
     */
    public static void showDisplayable(Displayable toShow, Displayable backscreen) {
        backStack.push(backscreen);
        display.setCurrent(toShow);
    }
    
    public static FunCanvasContactList getContactList() {
        
        if (contactList == null)  {
            
            ContactListLoader loader = getContactListLoader();
            loader.load();
            while (loader.isLoading()) {
                try {
                    Thread.sleep(250);
                } catch (InterruptedException ex) {
                    ex.printStackTrace();
                    Log.error("getContactList: " + ex.toString() );
                }
            }
        }
        return contactList;
    }
    
    /**
     * @return the current contactListLoader, or create one with empty message
     * if not yet initialized
     */
    private static ContactListLoader getContactListLoader() {
        
        if ( contactListLoader == null ){
            contactListLoader =  getContactListLoader(new Message());
        }
        return contactListLoader;
        
    }
    
    /**
     * return current contactlistloader with message set to the given one,
     * or create a new contaclistloader
     */
    public static ContactListLoader getContactListLoader(Message message) {
        
        if ( contactListLoader == null ){
            contactListLoader = new ContactListLoader(message);
        } else {
            contactListLoader.setMessage(message);
        }
        return contactListLoader;
    }
    
    private static Runnable getMessageWriter(Message message) {
        
        if ( messageWriter == null ) {
            messageWriter = new MessageWriter();
        }
        messageWriter.setMessage(message);
        
        return messageWriter;
        
    }
    
    private static SyncTaskManager getSyncTaskManager() {
        if ( syncTaskManager == null ) {
            syncTaskManager = new SyncTaskManager(messageManager, 2);
        } else {
            syncTaskManager.reset();
        }
        return syncTaskManager;
    }
    
    public static boolean checkContactListFull() {
        return (getContactManager().getContactCount()>=
                getContactManager().getDefaultContactNumber()
                &&getContactManager().getDefaultContactNumber()!=-1);
    }
    
    /**
     * return the message related message list
     * @param message is the reference message
     */
    public static FunCanvasMessageList getMessageList(Message message) {
        String folderName = message.getParent().getName();
        if (folderName.equals(Store.INBOX)) {
            return getInboxMessageList();
        } else if (folderName.equals(Store.OUTBOX)) {
            return getOutboxMessageList();
        } else if (folderName.equals(Store.DRAFTS)) {
            return getDraftMessageList();
        } else if (folderName.equals(Store.SENT)) {
            return getSentMessageList();
        }
        return null;
    }
    
    public static Drawer getDrawer() {
        if (drawer==null) {
            drawer = new RoundedDrawer();
        }
        return drawer;
    }
    
}


/**
 * private class for writing message to folder
 */

class MessageWriter implements Runnable {
    
    private  Message message = null;
    
    public MessageWriter() {
    }
    
    public void setMessage(Message m) {
        message = m;
    }
    
    public void run() {
        if (message != null) {
            UIController.messageManager.updateMessageFlags(message, message.getFlags());
            try {
                message.getParent().appendMessage(message);
            } catch (MailException ex) {
                Log.error("threadedWriteMessageToFolder: ", ex.toString() );
                ex.printStackTrace();
            } finally {
                message = null;
            }
        }
    }
    
}

