/*
 * Copyright (C) 2006-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 */

package com.funambol.mailclient.ui.view;

import javax.microedition.lcdui.Graphics;

/**
 *
 * simple drawing interface, implements some methods for drawing simple items
 * TODO: check if an (abstract) class may be better
 *
 */
public interface Drawer {
    
    /**
     * draw the canvas background in the given region
     *
     * @param g a graphics object
     * @param w the width of the region to be drawn
     * @param h the height of the region to be drawn
     */
    void drawCanvasBackground(Graphics g, int w, int h);
    
    
    
   /**
     * draw the title background in the given region
     *
     * @param g a graphics object
     * @param w the width of the region to be drawn
     * @param h the height of the region to be drawn
     */
    void drawTitleBackground(Graphics g, int w, int h);
    
    /**
     * draw a background in the given region
     *
     *@param g a graphics object
     *@param w the width of the region to be drawn
     *@param h the height of the region to be drawn
     *@selected is the item selected or not?
     */
    void drawBackground(Graphics g, int w, int h, boolean selected);
    
    /**
     * draw a string in the given graphics at the given coordinates with
     * layout set to Graphics.TOP|Graphics.LEFT
     * @param string the string to be drawn
     *@param graphics the graphics object
     *@param x the x coordinate
     *@param y the y coordinate
     *@param selected if true draw the string with highlighted font
     *
     */
    void drawString(String string, Graphics graphics, int x, int y, boolean selected);
    
    /**
     * draw a string in the given graphics at the given coordinates with
     * layout set to Graphics.TOP|Graphics.LEFT
     * @param string the string to be drawn
     *@param graphics the graphics object
     *@param x the x coordinate
     *@param y the y coordinate
     *@param layout the layout described with combination of the Graphics layout constants
     * @param selected if true draw the string with highlighted font
     *
     */
    void drawString(String string, Graphics graphics, int x, int y, int layout, boolean selected);
    
    
    /**
     * draw a string using bold font in the given graphics at the given coordinates with
     * layout set to Graphics.TOP|Graphics.LEFT
     * @param string the string to be drawn
     * @param graphics the graphics object
     * @param x the x coordinate
     * @param y the y coordinate
     * @param selected if true draw the string with highlighted font
     *
     */
    void drawBoldString(String string, Graphics graphics, int x, int y, boolean selected);
    
    /**
     * @returns the current graphicaltheme
     */
    GraphicalTheme getGraphicalTheme();
    
}
