/*
 * Copyright (C) 2006-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */


package com.funambol.mailclient.ui.view;

import com.funambol.mail.Address;
import com.funambol.mail.MailException;
import com.funambol.mail.Message;
import com.funambol.mailclient.Cache;
import com.funambol.mailclient.cm.Contact;
import com.funambol.mailclient.cm.ContactManagerException;
import com.funambol.mailclient.sm.SyncClient;
import com.funambol.mailclient.ui.controller.UIController;
import com.funambol.mailclient.ui.controller.UIThreadPoolMonitor;
import com.funambol.mailclient.ui.utils.UiUtils;
import com.funambol.mailclient.loc.Localization;
import com.funambol.util.Log;
import com.funambol.util.StringUtil;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Vector;
import javax.microedition.lcdui.Canvas;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;
import javax.microedition.rms.RecordStoreException;

public class FunCanvasContactList extends FunCanvasList implements Runnable {
    
    /**
     * none is composed by Address.TO|Address.CC|Address.BCC
     **/
    public static final int NONE=Address.TO|Address.CC|Address.BCC;
    
    // margin constants
    private static final int V_MARGIN =3;
    private static final int H_MARGIN =5;
    
    private static Font labelFont;
    private static Font titleFont;
    
    private Drawer drawer;
    
    private Font font;
    
    private GraphicalTheme graphicalTheme;
    
    private ContactItem[] contactItems;
    
    private String[] searchString;
    
    /**
     * the message from wich the recipients are shown and to which the
     * recipients are set
     */
    private Message message;
    
    /**
     * the number to be shown in the upper left corner,
     */
    private int contactCount;
    
    
    private Command syncCommand;
    
    private Command searchCommand;
    
    private Command addContactCommand;
    
    private Command okCommand;
    
    private Command selectCommand;
    
    private Command editContactCommand;
    
    private Command deleteContactCommand;
    
    private Command saveContactCommand;
    
    private Command resetContactsCommand;
    
    
    /**
     * the contact loader. we need only one of this, sort of singleton.
     */
    private RMSContactLoader rmsContactLoader;
    
    
    public FunCanvasContactList() {
        this( new Message() );
        
    }
    
    /** Creates a new instance of FunCanvasContactList with the given message */
    public FunCanvasContactList(Message message) {
        super();
        this.message = message;
        this.drawer = UIController.getDrawer();
        this.font = drawer.getGraphicalTheme().getDefaultFont();
        this.labelFont = drawer.getGraphicalTheme().getBoldFont();
        this.titleFont = drawer.getGraphicalTheme().getTitleFont();
        this.graphicalTheme = drawer.getGraphicalTheme();
        setContactCount(getRmsCount());
        setTitle(getDefaultTitle());
        setCommandListener(this);
        initializeContactList();
//        keyMapper = new KeyMapper();
    }
    
    
    
    /**
     * reset the contact list using the given message.
     * @param message the message
     */
    public void resetContactList(Message message) {
        this.message = message;
        setContactCount(getRmsCount());
        setTitle(getDefaultTitle());
        contactItems=null;
        searchString = null;
        initializeContactList();
    }
    /**
     * @return the default title
     */
    public String getDefaultTitle() {
        return Localization.getMessages().SELECT_RECIPIENTS;
    }
    
    int paintElement(int elementId, Graphics g) {
        int h = 0;
        //can be null if trying to paint during a sync
        //Log.debug("painting element " + elementId);
        if (contactItems[elementId]!=null) {
            g.translate(0,current_paint_h);
            h=contactItems[elementId].paint(g) ;
            g.translate(0, -current_paint_h);
        }
        return h;
    }
    
    
    
    
    void handleKey(int keyCode) {
        //  Log.debug("Key!");
        if(getGameAction(keyCode)==Canvas.LEFT) {
            //  Log.debug("Left!");
            if (contactItems[getActiveElement()].hasMultipleEmails()) {
                contactItems[getActiveElement()].toPreviousEmail();
            } else {
                scroll(-1);
            }
            repaint();
        } else if (getGameAction(keyCode) == Canvas.RIGHT ) {
            //   Log.debug("Right!");
            if (contactItems[getActiveElement()].hasMultipleEmails()) {
                contactItems[getActiveElement()].toNextEmail();
            } else {
                scroll(1);
            }
            repaint();
        }
        /*
            Log.debug("keycode= " + keyCode + ", gameAction= " + getGameAction(keyCode) +
                    ", keyname= " + getKeyName(keyCode) + " Char: " +  (char) keyCode);
            keyMapper.addKey(keyCode);
            searchString= keyMapper.getStrings();
         
         */
    }
    
    /**
     * empty the search string
     */
    public void emptySearchString() {
        
        String[] s = new String[1];
        s[0]="";
        setSearchString(s);
        
    }
    
    void initCommands() {
        
        okCommand =
                new Command(Localization.getMessages().AL_OK_COMMAND,
                UIController.COMMAND_TYPE,5);
        addContactCommand =
                new Command(Localization.getMessages().AL_ADD_CONTACT_COMMAND,
                UIController.COMMAND_TYPE,30);
        searchCommand =
                new Command(Localization.getMessages().AL_SEARCH_COMMAND,
                UIController.COMMAND_TYPE, 40);
        syncCommand =
                new Command(Localization.getMessages().AL_SYNC_COMMAND,
                UIController.COMMAND_TYPE, 40);
        resetContactsCommand=
                new Command(Localization.getMessages().RESET_CONTACTS_COMMAD,
                UIController.COMMAND_TYPE, 45);
        selectCommand =
                new Command(Localization.getMessages().SELECT_COMMAND,
                UIController.COMMAND_TYPE,20);
        editContactCommand =
                new Command(Localization.getMessages().EDIT_COMMAND_LABEL,
                UIController.COMMAND_TYPE, 50);
        deleteContactCommand =
                new Command(Localization.getMessages().DELETE_COMMAND_LABEL,
                UIController.COMMAND_TYPE, 60);
        saveContactCommand  =
                new Command(Localization.getMessages().ABF_SAVE_CONTACT_COMMAND,
                UIController.COMMAND_TYPE, 70);
        
    }
    
    void addCommands() {
        
        addCommand(UIController.cancelCommand);
        
        // Do not show "Ok" until a contact is selected in TO
        // Now "addCommand(okCommand);" is in toNextState() and in addAddress
        // methocs in the ContactItem class
        //addCommand(okCommand);
        
        addCommand(addContactCommand);
        addCommand(searchCommand);
        
        if (!SyncClient.isBusy()){
            addCommand(syncCommand);
            addCommand(resetContactsCommand);
            
        }
    }
    
    
    /**
     * called when the contact list is set to a non empty list
     * removes item-specific commands
     */
    void removeItemCommands() {
        removeCommand(selectCommand);
        removeCommand(editContactCommand);
        removeCommand(deleteContactCommand);
        setFireCommand(null);
    }
    /**
     * called when the contact list is set to a non-empty list.
     * adds item-specific commands
     */
    void addItemCommands() {
        addCommand(selectCommand);
        addCommand(editContactCommand);
        addCommand(deleteContactCommand);
        setFireCommand(selectCommand);
    }
    
    void paintBackground(Graphics graphics) {
        drawer.drawCanvasBackground(graphics, getWidth(), getHeight());
    }
    
    /**
     * paint the title
     * @return the height of the painted area
     */
    public int paintTitle(Graphics g) {
        
        //  Log.debug("paint title!");
        
        String s;
        String title = getTitle();
        
        // keep this because otherwhise some motorola phones may throw a
        // nullpointer exception
        //TODO: check if still true when using canvas!
        if (title == null){
            //    Log.debug("title null... setting to empty string");
            title ="";
        }
        int num = Math.max(elements.length, contactCount);
        if (elements.length > 0) {
            s = " " + (getActiveElement()+1) + "/" + num;
        } else if (num == 0){
            s = " (Empty)";
        } else {
            s = " " + num;
        }
        int sWidh = titleFont.stringWidth(s);
        
        String shortTitle = UiUtils.cutString(title,getWidth()-sWidh,titleFont);
        drawer.drawTitleBackground(g,getWidth(), titleFont.getHeight());
        g.setFont(titleFont);
        g.setColor(drawer.getGraphicalTheme().getTitleColor());
        //not using drawTitle because we need to draw also the number of messages
        g.drawString(shortTitle,0,0,Graphics.TOP | Graphics.LEFT);
        g.drawString(s, getWidth(),0,Graphics.TOP | Graphics.RIGHT);
        return drawer.getGraphicalTheme().getTitleFont().getHeight();
    }
    
    /**
     * @return true if element is activable.
     *
     */
    boolean isActivable(int elementId) {
        return isVisible(elementId);
    }
    
    /**
     * @return true if element must be shown, i.e. if is selected or if
     * valid for the search performed ( if the contact first name, last name or visiblename
     * starts for one of the given strings, the element is visible )
     */
    boolean isVisible(int elementId) {
        //return true if item is a recipient
        
        
        // FIXME (ugly code section ahead)
        if (contactItems==null) //avoid a nullpointer exception, do not remove!
            return false;
        if (elementId >= contactItems.length || elementId < 0)  //avoid outofbound
            return false;
        if (contactItems[elementId] == null)
            return false;
        // end ugly code
        
        if (searchString == null || contactItems[elementId].getAddress() != null) {
            return true;
        }
        Contact c= contactItems[elementId].getContact();
        
        for (int i=0; i<searchString.length; i++) {
            
            if ( smartStartsWith( c.getVisibleName(), searchString[i] ) ||
                    smartStartsWith( c.getFirstName(), searchString[i] ) ||
                    smartStartsWith( c.getLastName(), searchString[i] )) {
                return true;
            }
            
        }
        return false;
    }
    
    
    /**
     * return true if tocheck starts with start (case insensitive)
     */
    boolean smartStartsWith(String toCheck, String start) {
        
        if (StringUtil.isNullOrEmpty(toCheck)) {
            return false;
        }
        return toCheck.toLowerCase().startsWith(start.toLowerCase());
        
    }
    
    /**
     * sync the contacts
     */
    public void run() {
        
        UIController.synchContacts();
        initializeContactList();
        this.setTitle(this.getDefaultTitle());
        
    }
    
    public void commandAction(Command command, Displayable displayable) {
        
        //   Log.debug(this, "command pressed: "+command.getLabel());
        if (command == UIController.cancelCommand) {
            
            //if this is not null, we reset the title
            if (UIController.composeMessageScreen!=null) {
                UIController.getComposeMessageScreen().setDefaultTitle();
            }
            UIController.showBackScreen();
        } else if (command == searchCommand) {
            UIController.display.setCurrent(new SearchAddressForm(this));
        } else if (command == okCommand) {
            updateMessageRecipients();
            UIController.updateComposeMessageScreen(message);
            UIController.showComposeMessageScreen(
                    null, UIController.getComposeMessageScreen().subject);
        } else if (command == addContactCommand) {
            if (UIController.checkContactListFull()) {
                UIController.showErrorAlert(
                        Localization.getMessages().AB_LIMIT_ERROR_MSG);
                return;
            }
            updateMessageRecipients();
            UIController.showAddContactScreen(UIController.display.getCurrent(), this);
        } else if (command == syncCommand ) {
            this.setTitle(Localization.getMessages().FAL_SYNC_IN_PROGRESS);
            UIController.getThreadPool().startThread(this);
        }
        
        //item command
        else if (command == selectCommand) {
            contactItems[getActiveElement()].toNextState();
        } else if (command == editContactCommand) {
            UIController.showEditContactScreen(this,  contactItems[getActiveElement()].getContact() , this, false);
        } else if (command == deleteContactCommand) {
            UIController.deleteContact(contactItems[getActiveElement()].getContact());
          
            removeContactItem(getActiveElement());
            //setContactCount(getRmsCount());
            
            Contact[] contacts = new Contact[contactItems.length];
            int lim = Cache.CONTACT_SNAPSHOT_SIZE < contactItems.length ? 
                Cache.CONTACT_SNAPSHOT_SIZE : contactItems.length;
            for(int i=0; i < lim; i++){
                contacts[i] = contactItems[i].getContact();
            }
            try{
                Cache.saveContactSnapShot(Cache.VCARD, contacts);
            } catch (ContactManagerException ex) {
                Log.error(this, "Error loading contacts from rms");
                ex.printStackTrace();
            }  catch (RecordStoreException ex) {
                Log.error(this, "Error saving contact snapshot to rms");
                ex.printStackTrace();
            }  catch (IOException ex) {
                Log.error(this, "Error saving contact snapshot to rms");
                ex.printStackTrace();
            }
            
            initializeContactList();
      
        } else if (command == resetContactsCommand) {
            
            FunModalPopup popup =
                    new FunModalPopup(Localization.getMessages().RESET_REQUEST,
                    Localization.getMessages().RESETCONTACTS_MESSAGE,
                    new ResetContactsPopupAction());
            UIController.showModalPopup(popup, this);
            
            
        }
        
    }
    
    private void updateMessageRecipients() {
        message.removeAllRecipients(Address.TO);
        message.removeAllRecipients(Address.CC);
        message.removeAllRecipients(Address.BCC);
        
        Address address;
        
        for (int i=0; i< elements.length; i++) {
            ContactItem item = (ContactItem)elements[i];
            address=item.getAddress();
            if (address != null) {
                //    Log.debug("Address: " + address.getEmail());
                try {
                    message.addRecipient(address);
                    
                } catch (MailException me) {
                    Log.error(this , "mail exception updateMessageRecipients:" + me.toString());
                    me.printStackTrace();
                }
            }
        }
    }
    
    protected void initializeContactList() {
        Enumeration e;
        Vector contactsFromMemory= new Vector();
        setTitle(Localization.getMessages().LOADING_CONTACTS);
        Address[] messageAddresses = getMessageAddresses();
        
        //this will store the contactItems from the message
        ContactItem[] contactItemsFromMessage =
                new ContactItem[messageAddresses.length];
        
        if (messageAddresses.length == 0) {
            //we can use the cache!
            //Log.debug(this,"using cache");
            try {
                // Log.debug("Calling cache.getContactSnapshot");
                Contact [] cList = Cache.getContactSnapShot(Cache.VCARD);
                ContactItem[] cItemList =  new ContactItem[cList.length];
                for (int i = 0; i<cList.length; i++) {
                    cItemList[i] = new ContactItem(cList[i],i);
                    setContactItemList(cItemList);
                }
            } catch (Exception ex) {
                Log.error("Error getting contact list from cache");
                ex.printStackTrace();
            }
            UIController.getThreadPool().startThread(getRMSContactLoader());
            
        } else {
            Log.debug(this,"not using cache");
            int found=0;
            
            try {
                if(contactItems != null){
                    boolean add = false;
                    for (int i=0; i< contactItems.length; i++) {
                        add = true;

                        for (int y=0; y< messageAddresses.length; y++){
                            if((contactItems[i]).getContact().getDefaultEmail().equals(messageAddresses[y].getEmail()) &&
                                (contactItems[i]).getContact().getVisibleName().equals(messageAddresses[y].getVisibleName())){
                                add = false;
                            }
                        }
                        if(add){
                            contactsFromMemory.addElement((contactItems[i]).getContact());
                        }
                    }
                }else{
                    e = UIController.getContactManager().getContactList();
                    while (e.hasMoreElements()) {
                        Contact c = (Contact) e.nextElement();
                        if (found < messageAddresses.length) {

                            AddressPointer addressPointer =
                                    searchAddress(c, messageAddresses);
                            if (addressPointer.index >= 0) {
                                found++;
                                contactItemsFromMessage[found-1] =
                                        new ContactItem(c,found-1, addressPointer.addressType,
                                        addressPointer.emailType);
                                messageAddresses[addressPointer.index] = null;
                            } else {
                                contactsFromMemory.addElement(c);
                            }
                        } else {
                            contactsFromMemory.addElement(c);
                        }
                    }
                }
                
                for (int i=0; i< messageAddresses.length; i++) {
                    if (messageAddresses[i]!= null) {
                        
                        Address address = messageAddresses[i];
                        Contact contact = new Contact(address.getVisibleName(), address.getEmail());
                        ContactItem cItem = new ContactItem(contact, found, address.getType(), Contact.DEFAULT_EMAIL);
                        contactItemsFromMessage[found] =cItem;
                        found++;
                    }
                }
                //ContactItem[]contactItemsFromMemory = new ContactItem[contactsFromMemory.size()];
                e = contactsFromMemory.elements();
                
                ContactItem[] contactItemList = new ContactItem[contactItemsFromMessage.length + contactsFromMemory.size()];
                System.arraycopy(contactItemsFromMessage,0,contactItemList,0, contactItemsFromMessage.length);
                int i=0;
                while (e.hasMoreElements()) {
                    contactItemList[found+i] = new ContactItem((Contact)e.nextElement(),found+i);
                    i++;
                }
                
                // contactList = new Contact[found + contactsFromMemory.size()];
                // Contact[] contactsFromMemory = new Contact[contactsFromMemory.size()];
                // contactsFromMemory.copyInto(contactsFromMemory);
                // contactItemsFromMessage
                // contactsFromMemory.copyInto(contactList);
                setContactItemList(contactItemList);
                enableDisableOKCommand();
                } catch (ContactManagerException ex) {
                Log.error(this, "Error getting contact list");
                ex.printStackTrace();
            } finally {
                setTitle(getDefaultTitle());
            }
            
            
        } //end of cache block
        
    }
    
    protected  Address[] getMessageAddresses() {
        Address[] toAddressList ;
        Address[] ccAddressList ;
        Address[] bccAddressList ;
        
        try {
            toAddressList=message.getTo();
            ccAddressList=message.getCc();
            bccAddressList=message.getBcc();
        } catch (MailException ex) {
            ex.printStackTrace();
            Log.error("getMessageAddresses: "+ex.toString());
            return new Address[0];
        }
        
        int size = toAddressList.length
                + ccAddressList.length
                + bccAddressList.length;
        
        Address[] messageAddresses=new Address[size];
        
        //     Log.debug(this,"created message Addressess vector with lenght "
        //           + messageAddresses.length);
        
        for (int i=0; i < toAddressList.length; i++) {
            messageAddresses[i]=toAddressList[i];
        }
        for (int i=0; i<ccAddressList.length; i++) {
            messageAddresses[toAddressList.length + i] =
                    ccAddressList[i];
        }
        for (int i=0; i<bccAddressList.length; i++) {
            messageAddresses[toAddressList.length
                    +ccAddressList.length + i] =
                    bccAddressList[i];
        }
        return messageAddresses;
    }
    
    private AddressPointer searchAddress(Contact contact, Address[] messageAddresses) {
        Log.trace(this, "searchAddress" + contact.getVisibleName());
        AddressPointer pointer = new AddressPointer(-1,0,0);
        Address a;
        for (int i=0; i<messageAddresses.length; i++) {
            a=messageAddresses[i];
            if (a!=null) {
                String mail = a.getEmail().toLowerCase().trim();
                if ( mail.equals( contact.getDefaultEmail().toLowerCase().trim() ) )  {
                    pointer = new AddressPointer(i,a.getType(),Contact.DEFAULT_EMAIL);
                } else if ( a.getEmail().equals( safeTrimLowercase(contact.getEmail_2()) ) ) {
                    pointer = new AddressPointer(i,a.getType(),Contact.SECONDARY_EMAIL);
                } else if ( a.getEmail().equals( safeTrimLowercase(contact.getEmail_3()) ) ) {
                    pointer = new AddressPointer(i,a.getType(),Contact.TERTIARY_EMAIL);
                }
            }
        }
        return pointer;
    }
    
    
    private String safeTrimLowercase(String in) {
        
        if (StringUtil.isNullOrEmpty(in)) {
            return "";
        } else {
            return in.toLowerCase().trim();
        }
    }
    
    
    protected void setContactItemList(ContactItem[] contactItems) {
        
        super.setElements(contactItems);
        this.contactItems = contactItems;
        setContactCount(getRmsCount());
        
        repaint();
    }
    
    public Contact getActiveContact() {
        return contactItems[getActiveElement()].getContact();
    }
    /**
     * set the search string array to the given array.
     *
     * @param string an array containing the strings to be used by
     * isVisible function ( if the contact first name, last name or visiblename
     * starts for one of the given strings, the element is visible )
     */
    public void setSearchString(String[] string) {
        this.searchString = string;
        //this find the first activable element
        rewindList();
        repaint();
        
    }
    
    void addAddress(Address address) {
        try {
            message.addRecipient(address);
            addCommand(okCommand);
        } catch (MailException ex) {
            ex.printStackTrace();
            Log.error("Unable to add recipient "+address.getEmail() + "to message");
        }
        this.initializeContactList();
        
    }
    
    void resetActiveElementHeight() {
        contactItems[getActiveElement()].resetHeight();
    }
    
    protected void enableDisableOKCommand() {
        
        for (int i=0; i< contactItems.length; i++) {
            if (contactItems[i].state != NONE){
                addCommand(okCommand);
                break;
                
            } else {
                removeCommand(okCommand);
            }
            
        }
    }
    
    private int getRmsCount() {
        return  UIController.getContactManager().getContactCount();
    }
    
    /**
     * set the contact count to be shown in the upper left corner
     * @param count the contact count to be shown in the upper left corner,
     * if a negative number is passed, the contact count used will be the
     * total number of contacts in rms
     */
    public void setContactCount(int count) {
        if (count < 0 ) count = getRmsCount();
        contactCount = count;
        
    }
    
   
    
    /**
     * enable or disable the sync command and the reset contacts command
     * @param enable if true enable the command, if false disable the command
     */
    public void enableSyncCommand(boolean enable){
        if (enable) {
            addCommand(syncCommand);
            addCommand(resetContactsCommand);
        } else {
            removeCommand(syncCommand);
            removeCommand(resetContactsCommand);
        }
    }
    
    /**
     * if the given contact is in the list, it is set as the active element.
     * @param contact the contact that must be set as active element
     * 
     */
    public void activateElement(Contact contact){
        int id = -1;
        for (int i=0; i< contactItems.length; i++) {
            if ((contactItems[i].getContact().getDefaultEmail()).equals(contact.getDefaultEmail()) ){
                id = i;// contactItems[i].getElementID();
                break;
            }
        }
        if (id > 0) {
            setActiveElement(id);
        }

    }
    
    /**
     * Scan the contact list looking for a contact with given 
     * email addres as the default email
     * @param email is the given contact mail
     * @return the Contact with the give email address , or null if
     * the contact is not present
     */
    public Contact exists(String  email){
        for (int i=0; i< contactItems.length; i++) {
            if ((contactItems[i].getContact().getDefaultEmail()).equals(email) ){
                return contactItems[contactItems[i].getElementID()].getContact();
            }
        }
        return null;
    }
    
    
    /**
     * remove the specified contat item from the 
     * contactitem vector. 
     * @param toRemove the position of the item 
     * to be removed in the elemnts vector 
     */
    private void removeContactItem(int toRemove){
        
        
        Vector helpVector = new Vector();
        for(int i=0; i< elements.length; i++){
            if(i!=toRemove){
                helpVector.addElement(elements[i]);
            }
        }
        contactItems = new ContactItem[elements.length -1];
        helpVector.copyInto(contactItems);
    }
  
    
    /**
     * get the rmscontact loader or create a new one if null
     *
     * @return the rmsContactLoader
     */
    private Runnable getRMSContactLoader() {
        if (rmsContactLoader == null) {
            rmsContactLoader = new RMSContactLoader();
        }
        
        return rmsContactLoader;
        
    }
    
//
// -------------------- RMSContactLoader Class -----------------------
//
    
    /**
     * inner class that loads contacts from the RMS
     */
    
    class RMSContactLoader implements Runnable {
        
        public RMSContactLoader() {
            
        }
        
        public void loadContacts() {
            try {
                //Log.debug(this, "Starting to load contacts from RMS!");
                
                setTitle(Localization.getMessages().LOADING_CONTACTS);
                //Remove command because Contact ID from cache is not the same with
                //Contact ID from RMS
                removeItemCommands();
                int count = UIController.getContactManager().getContactCount();
                Enumeration e = UIController.getContactManager().getContactList();
                
                Contact[] contactList = new Contact[count];
                ContactItem[] newContactItemList = new ContactItem[count];
                
                int i=0;
                
                while (e.hasMoreElements()) {
                    Contact c = (Contact) e.nextElement();
                    contactList[i] = c;
                    newContactItemList[i] = new ContactItem(c,i);
                    i++;
                }
                
                
                // orrible fix here. this is needed for safe thread sync. I suppose.
                // the problem is that sometimes the enumeration seems to be
                // smaller than the number of items returned by getContactCount,
                // so we need this to avoid nullpointers
                if (i == count) {
                    setContactItemList(newContactItemList);
                    Cache.saveContactSnapShot(Cache.VCARD, contactList);
                } else {
                    //if somethig went wrong, we try again!
                    loadContacts();
                }
                
                
            } catch (ContactManagerException ex) {
                Log.error(this, "Error loading contacts from rms");
                ex.printStackTrace();
            }  catch (RecordStoreException ex) {
                Log.error(this, "Error saving contact snapshot to rms");
                ex.printStackTrace();
            }  catch (IOException ex) {
                Log.error(this, "Error saving contact snapshot to rms");
                ex.printStackTrace();
            } finally {
                
                setTitle(getDefaultTitle());
                addItemCommands();
                //     Log.debug(this, "end loading contacts from rms!");
            }
        }
        
        public void run() {
            loadContacts();
        }
    }
    
    
    
//
// ----------------------------- AddressPointer Class --------------------------
//
    
    
    /**
     * return an object with index, addresstype and emailtype.
     * this class is used when searching for rms contacts mathching the ones
     * in the message given.
     */
    class AddressPointer {
        int index;
        int addressType;
        int emailType;
        
        public AddressPointer(int index, int addressType, int emailType) {
            this.index = index;
            this.addressType = addressType;
            this.emailType = emailType;
        }
        
    }
    
    
//
// ----------------------------- ContactItem Class --------------------------
//
    
    /**
     * item class to paint a contact and handle recipent settings
     *
     *<p> a contactItem has a selected Email amongst the email addresses of the given
     * contact multiple email addresses, and a state that can be one of
     *  <ul>
     *   <li>NONE</li>
     *   <li>Address.TO</li>
     *   <li>Address.CC</li>
     *   <li>Address.BCC</li>
     *  </ul>
     */
    class ContactItem {
        
        /**
         * states can be Address.TO, Address.CC, Address.BCC, or FunContactItem.NONE
         */
        private int state=NONE;
        
        
        private Contact contact;
        
        private int id;
        
        private int height =0;
        
        private int maxLabelWidth;
        
        private int selectedEmailIndex = Contact.DEFAULT_EMAIL;
        
        
        /**
         * create a new ContactItem with given data and default state and email
         *
         * @param contact the contact
         * @param elementId the elementID of this contact, is used by the
         * ContactItem to check if this is the active element
         */
        
        public ContactItem(Contact contact, int elementId) {
            this(contact, elementId, NONE, Contact.DEFAULT_EMAIL);
        }
        
        /**
         * create a new ContactItem with given data
         *
         * @param contact the contact
         * @param elementId the elementID of this contact, is used by the
         * ContactItem to check if this is the active element
         * @param state one of Address.TO Address.CC Address.BCC or
         * FunCanvasContactList.NONE
         *@param selectedEmail one of Contact.DEFAULT, Contact.SECONDARY_EMAIL
         * or Contact.TERTIARY_EMAIL
         */
        public ContactItem(Contact contact, int elementId,  int state, int selectedEmail) {
            
            
            this.contact  = contact;
            this.id = elementId;
            this.state = state;
            this.selectedEmailIndex = selectedEmail;
            
            
        }
        /**
         * reset the heigth of this contactItem.
         * used so that height is recalculated at the next paint call.
         */
        public void resetHeight() {
            this.height = 0;
        }
        
        public Contact getContact() {
            return contact;
        }
        
        public int paint(Graphics graphics) {
            
            int h = calculateHeight();
            int w = getWidth();//- H_MARGIN;
            
            drawer.drawBackground(graphics,getWidth(),h, isActive());
            
            if (isActive()) {
                graphics.setColor(graphicalTheme.getHighlightedForegroundColor());
            } else {
                graphics.setColor(graphicalTheme.getForegroundColor());
            }
            
            // translate the graphics to position the icon
            
            //clip the rect where we'll draw the icon
            //graphics.setClip(0,0,getMaxLabelWidth()/2,font.getHeight()/3 - 2 * V_MARGIN);
            graphics.translate( w -15, 1);
            //graphics.setClip(w-15,1,12,8);
            graphics.setClip(0,0,12,8);
            
            drawSwitchEmailIcon(graphics);
            
            graphics.translate( -w +15, -1);
            graphics.setClip( 0, 0, getWidth(), getHeight() );
            
            
            graphics.drawLine(getMaxLabelWidth()+2*H_MARGIN,0,getMaxLabelWidth()+2*H_MARGIN,h);
            graphics.setClip(H_MARGIN,V_MARGIN, getMaxLabelWidth(),h-V_MARGIN);
            graphics.translate(H_MARGIN,V_MARGIN);
            drawLabel(graphics,w,h,isActive());
            graphics.translate(-H_MARGIN, -V_MARGIN);
            graphics.setClip(0,0,w,h);
            graphics.setClip(getMaxLabelWidth()+3*H_MARGIN,V_MARGIN,w-(getMaxLabelWidth()+3*H_MARGIN), h);
            graphics.translate(getMaxLabelWidth()+3*H_MARGIN,V_MARGIN);
            drawName(graphics,w,h,isActive());
            
            
            
            //back to the original graphic
            graphics.translate(-getMaxLabelWidth()-3*H_MARGIN,-V_MARGIN);
            graphics.setClip(0,0,getWidth(),getHeight());
            
            return h;
        }
        
        /**
         * draws the label ("to:", "cc:","bcc:")
         */
        private void drawLabel(Graphics graphics, int w, int h, boolean b) {
            if (isActive())
                graphics.setColor(graphicalTheme.getHighlightedForegroundColor());
            else
                graphics.setColor(graphicalTheme.getForegroundColor());
            
            graphics.setFont(labelFont);
            graphics.drawString(getAddressTypeLabel(),0,0,Graphics.TOP|Graphics.LEFT);
            
        }
        
        private String getAddressTypeLabel() {
            
            switch (state) {
                
                case Address.TO:
                    return Localization.getMessages().TO_LABEL.trim();
                    
                case Address.CC:
                    return Localization.getMessages().CC_LABEL.trim();
                    
                case Address.BCC:
                    return Localization.getMessages().BCC_LABEL.trim();
                    
                default: //and NONE
                    return "";
            }
            
        }
        /**
         * change the state of the item the the next one. <br>
         * sequence is NONE -> TO -> CC -> BCC -> NONE ...
         */
        public void toNextState() {
            
            //Log.debug(this, "To nextState! ");
            
            switch (state) {
                case NONE:
                    state=Address.TO;
                    enableDisableOKCommand();
                    break;
                case Address.TO:
                    state=Address.CC;
                    break;
                case Address.CC:
                    state=Address.BCC;
                    break;
                case Address.BCC:
                    state=NONE;
                    enableDisableOKCommand();
                    break;
                default:state=NONE;
            }
            repaint();
        }
        
        
        /**
         * cycle to the next email changing the value of selectedImailIndex
         */
        private void toNextEmail() {
            //we know we have at least 2 emails, and we have defaultemail
            switch (selectedEmailIndex) {
                
                case Contact.DEFAULT_EMAIL:
                    if (! StringUtil.isNullOrEmpty( contact.getEmail_2()) ) {
                        selectedEmailIndex = Contact.SECONDARY_EMAIL;
                    } else {
                        selectedEmailIndex = Contact.TERTIARY_EMAIL;
                    }
                    break;
                    
                case Contact.SECONDARY_EMAIL:
                    if (! StringUtil.isNullOrEmpty( contact.getEmail_3()) ) {
                        selectedEmailIndex = Contact.TERTIARY_EMAIL;
                    } else {
                        selectedEmailIndex = Contact.DEFAULT_EMAIL;
                    }
                    break;
                    
                default: //tertiary or unknown cases
                    selectedEmailIndex = Contact.DEFAULT_EMAIL;
                    break;
            }
        }
        
        
        /**
         * cycle to the previous email changing the value of selectedImailIndex
         */
        private void toPreviousEmail() {
            
            switch (selectedEmailIndex) {
                
                case Contact.DEFAULT_EMAIL:
                    if (! StringUtil.isNullOrEmpty( contact.getEmail_3()) ) {
                        selectedEmailIndex = Contact.TERTIARY_EMAIL;
                    } else {
                        selectedEmailIndex = Contact.SECONDARY_EMAIL;
                    }
                    break;
                    
                case Contact.TERTIARY_EMAIL:
                    if (! StringUtil.isNullOrEmpty( contact.getEmail_2()) ) {
                        selectedEmailIndex = Contact.SECONDARY_EMAIL;
                    } else {
                        selectedEmailIndex = Contact.DEFAULT_EMAIL;
                    }
                    break;
                    
                default: //secondary and unknown
                    selectedEmailIndex = Contact.DEFAULT_EMAIL;
                    break;
            }
        }
        
        /**
         * draws the visiblename
         */
        private void drawName(Graphics graphics, int w, int h, boolean b) {
            if (isActive()) {
                graphics.setColor(graphicalTheme.getHighlightedForegroundColor());
            } else {
                graphics.setColor(graphicalTheme.getForegroundColor());
            }
            graphics.setFont(font);
            //TODO: use something to avoid cutstring calls every time this function
            //is called
            drawer.drawString(UiUtils.cutString(getName(),graphics.getClipWidth(),font),graphics,0,0,isActive());
            if ( hasMultipleEmails() ) {
                drawer.drawString(UiUtils.cutString(getSelectedEmail(),graphics.getClipWidth(),font),graphics,H_MARGIN, font.getHeight(),isActive());
            }
        }
        
        /**
         * @return the selected email as a string
         */
        private String getSelectedEmail() {
            switch (selectedEmailIndex) {
                case Contact.DEFAULT_EMAIL:
                    return contact.getDefaultEmail();
                case Contact.SECONDARY_EMAIL:
                    return contact.getEmail_2();
                case Contact.TERTIARY_EMAIL:
                    return contact.getEmail_3();
                default:
                    return contact.getDefaultEmail();
                    
            }
        }
        
        
        private String getName() {
            return contact.getVisibleName();
        }
        
        
        private int getMaxLabelWidth() {
            
            if (maxLabelWidth==0) {
                int w1=labelFont.stringWidth(Localization.getMessages().TO_LABEL.trim());
                int w2=labelFont.stringWidth(Localization.getMessages().CC_LABEL.trim());
                int w3=labelFont.stringWidth(Localization.getMessages().BCC_LABEL.trim());
                maxLabelWidth= Math.max( w1, Math.max(w2,w3));
            }
            
            return maxLabelWidth;
            
        }
        
        private void drawSwitchEmailIcon(final Graphics graphics) {
            
            if ( hasMultipleEmails() ) {
                //graphics.setColor(0xff3333);
                graphics.fillTriangle(
                        graphics.getClipWidth() / 2 + 2 , 0    ,
                        graphics.getClipWidth(), graphics.getClipHeight() / 2,
                        graphics.getClipWidth() / 2 + 2 , graphics.getClipHeight()
                        );
                
                graphics.fillTriangle(
                        graphics.getClipWidth() / 2 - 2 , 0    ,
                        0, graphics.getClipHeight() / 2,
                        graphics.getClipWidth() / 2 - 2  , graphics.getClipHeight()
                        );
            }
        }
        
        
        private int calculateHeight() {
            if (height==0) {
                height= V_MARGIN;
                height+=Math.max(font.getHeight(), labelFont.getHeight());
                height+=V_MARGIN;
                if ( hasMultipleEmails() ) {
                    height += font.getHeight();
                }
            }
            return height;
        }
        
        private boolean hasMultipleEmails() {
            if ( StringUtil.isNullOrEmpty( contact.getEmail_2() ) &&
                    StringUtil.isNullOrEmpty( contact.getEmail_3() ) )  {
                return false;
            } else {
                return true;
            }
        }
        
        /**
         * @return the address or null if no selection has been done
         */
        public Address getAddress() {
            
            if (state==NONE)
                return null;
            else {
                //     Log.debug(this, "returning address for " + contact.getVisibleName() + " with state " + getAddressTypeLabel());
                try {
                    return contact.getAddress(state, selectedEmailIndex);
                } catch (MailException ex) {
                    //TODO: better exception handling and localization
                    //UIController.showErrorAlert("Error getting Address [this string need localization]");
                    Log.error(this, "Error getting Address from contact item");
                    ex.printStackTrace();
                    return null;
                }
            }
        }
        
        private boolean isActive(){
            //   Log.debug("active = " + activeElement + " id = " + id);
            return this.id == getActiveElement();
        }
        
        public int getElementID(){
            return id;
        }
        
        
    }
    
    
    //---------------------- ResetContactsPopupAction Class -------------------//
    
    /**
     * popupaction to reset contacts
     */
    public class ResetContactsPopupAction implements PopupAction {
        public ResetContactsPopupAction() {
        }
        
        public void cancel() {
            UIController.showBackScreen();
        }
        
        public void confirm() {
            // disableSyncCommand(true);
            // UIController.enableSyncCommands(false);
            UIController.showInboxScreen();
            UIController.resetContacts();
            // UIController.enableSyncCommands(true);
        }
    }
    
}
