/*
 * Copyright (C) 2006-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 */


/**
 * FunMessageCanvasList.java
 */

package com.funambol.mailclient.ui.view;

import com.funambol.mail.MailException;
import com.funambol.mail.Message;
import com.funambol.mail.MessageFlags;
import com.funambol.mail.Store;
import com.funambol.mail.StoreFactory;
import com.funambol.mailclient.config.ConfigManager;
import com.funambol.mailclient.mm.FolderInfo;
import com.funambol.mailclient.mm.MessageInfo;
import com.funambol.mailclient.mm.MessageManager;
import com.funambol.mailclient.ui.controller.Theme;
import com.funambol.mailclient.ui.controller.UIController;
import com.funambol.mailclient.ui.utils.UiUtils;
import com.funambol.util.Log;
import com.funambol.util.MailDateFormatter;
import java.util.Calendar;
import java.util.Vector;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;

public class FunCanvasMessageList extends  FunCanvasList implements CommandListener {
    /**
     * string to use when cutting a long string
     */
    private static final String CONTINUE_STRING="..";
    
    /**
     * the margin
     */
    private static final int MARGIN=3;
    
    /**
     * the MessageInfo list
     */
    // private CanvasMessageItem[] messageList;
    
    private Vector messageList;
    
    private static Font font;
    private static Font boldFont;
    private static Font titleFont;
    private static Drawer drawer;
    
    
    
    //private static GraphicalTheme;
    
    
    //-------------------------------------------------------------- Constructor
    
    /**
     * create a new FunCanvasMessageList
     */
    public FunCanvasMessageList(MessageInfo [] elements) {
        super(elements);
        this.drawer = UIController.getDrawer();
        this.font = drawer.getGraphicalTheme().getDefaultFont();
        this.boldFont = drawer.getGraphicalTheme().getBoldFont();
        this.titleFont = drawer.getGraphicalTheme().getTitleFont();
        this.messageList = new Vector(ConfigManager.getMaxMessageNumber());
        resetTitle();
        initializeMessageItems();
        setCommandListener(this);
        
    }
    
    
    //------------------------------------------------------------------ Methods
    
    /**
     * set the messageinfo list, and repaint the message list
     */
    public void setMessageInfoList(MessageInfo[] msgInfoList ) {
        setElements(msgInfoList);
        
        initializeMessageItems();
        
        repaint();
    }
    
    /**
     * @return the num of messages
     **/
    public int getNumMessages() {
        return elements.length;
    }
    
    /**
     * @return the selected message info
     */
    public MessageInfo getSelectedMessageInfo() {
        if (getActiveElement()<elements.length && getActiveElement() >=0) {
            return (MessageInfo) elements[getActiveElement()];
        } else {
            return null;
        }
    }
    
    
    
    /**
     *  paint the element
     */
    protected int paintElement(int elementId, Graphics g) {
        //Log.info ("painting element " + elementId);
        g.translate(0,current_paint_h);
        
        int h = ((CanvasMessageItem)messageList.elementAt(elementId)).paint(g);
        
        g.translate(0, -current_paint_h);
        
        return h;
    }
    
    
    
    /**
     * create the messagelist from the array of elements
     */
    private void initializeMessageItems() {
        
        messageList.removeAllElements();
        
        //messageList.en
        for (int i=0;i<elements.length;i++){
            messageList.addElement(new CanvasMessageItem(
                    (MessageInfo) elements[i], i));
        }
    }
    /**
     * handle commandaction
     */
    public void commandAction(Command command, Displayable displayable) {
        
        if (command == UIController.backCommand) {
            UIController.showBackScreen();
        }
    }
    /**
     * paint the title and return the height of the painted area
     */
    public int paintTitle(Graphics g) {
        
        //   Log.debug("paint title!");
        
        String s;
        String title = getTitle();
        
        // keep this because otherwhise some motorola phones may throw a
        // nullpointer exception
        if (title == null){
            //Log.debug("title null... setting to empty string");
            title ="";
        }
        s = " ";
        if (elements.length > 0) {
            s += (getActiveElement() + 1) + "/";
        }
        if ( getDisplayedMessageCount() > 0 ) {
            s += getDisplayedMessageCount();
        } else {
            s =" "+ getDisplayedMessageCount();
        }
        int sWidh = titleFont.stringWidth(s);
        
        String shortTitle = UiUtils.cutString(title,getWidth()-sWidh-1,titleFont);
        drawer.drawTitleBackground(g,getWidth(), titleFont.getHeight());
        g.setFont(titleFont);
        g.setColor(drawer.getGraphicalTheme().getTitleColor());
        //not using drawTitle because we need to draw also the number of messages
        g.drawString(shortTitle,1,1,Graphics.TOP | Graphics.LEFT);
        g.drawString(s, getWidth(),0,Graphics.TOP | Graphics.RIGHT);
        return drawer.getGraphicalTheme().getTitleFont().getHeight();
    }
    
    /**
     * this method return the number of messages as shown in the title bar
     * it can be different from the number of elements in the message array
     * because messages can be dinamically loaded or hidden for some reason
     */
    protected int getDisplayedMessageCount() {
        //TODO: use the count of visible element instead of elements.length
        
        return elements.length;
    }
    
    /**
     * return the default title
     */
    public String getDefaultTitle() {
        return "";
    }
    
    /**
     * reset the title to the default title
     */
    public void resetTitle() {
        //    Log.debug(this, "resetting title to " + getDefaultTitle());
        setTitle(this.getDefaultTitle());
    }
    
    
    /**
     * add the given messageInfo to the messagelist or overwrite the message
     * if exists
     *
     * @param messageInfo the messageinfo to be added
     */
    public void addMessage(MessageInfo messageInfo) {
        //   Log.debug(this, "addMessage (messageinfo)");
        if(elements.length == 0) {
            // This is the first message.
            setMessageInfoList(new MessageInfo[] { messageInfo});
            return;
        }
        // Otherwise, insert the message in the right position.
        
        int pos = search(messageInfo.messageId);
        if (pos < 0 ) {
            // this means we have to add the message
            if (elements.length >= ConfigManager.getMaxMessageNumber()) {
                //we need to remove one element from the list to make room
                //for this new message
                for (int i = 0; i< elements.length; i++) {
                    if (messageInfo.received.getTime() >
                            ((MessageInfo)elements[i]).received.getTime() ) {
                        for(int j=elements.length-1; j>i; j--) {
                            move(j, j-1);
                        }
                        //we insert the new message here
                        assign(i, messageInfo);
                        break;
                    }
                }
            } else {
                // we do not need to remove messages, we have to simply add the
                // new one at the right position
                //#ifdef low_mem
//#                 UIController.freeMemory();      //XXX
                //#endif
                
                MessageInfo[] newMessageInfoList=new MessageInfo[elements.length+1];
                
                //security check
                if (elements.length != messageList.size()) {
                    Log.error("elements length different from message list, " +
                            "reloading from rms");
                    setMessageInfoList(
                            UIController.getSortedMessageInfo(
                            UIController.getFolderInfo(MessageManager.INBOX)
                            ));
                    repaint();
                    return;
                }
                
                // find the right position to add the message to
                int i;
                for (i=0; i<elements.length; i++) {
                    if (messageInfo.received.getTime() >
                            ((MessageInfo)elements[i]).received.getTime() ) {
                        break;
                    }
                    newMessageInfoList[i] = (MessageInfo) elements[i];
                }
                //add the new message
                newMessageInfoList[i] = messageInfo;
                
                if (i < messageList.size() ) {
                    // inseert the element in the middle of the messageList
                    messageList.insertElementAt(
                            new CanvasMessageItem(messageInfo, i), i);
                    
                    for (i++; i < messageList.size() ; i++) {
                        newMessageInfoList[i] = (MessageInfo) elements[i-1];
                        ((CanvasMessageItem)messageList.elementAt(i)).setId(i);
                        
                    }
                } else {
                    // append the element at the end of the messagelist
                    messageList.addElement(
                            new CanvasMessageItem(messageInfo, i));
                }
                
                setElements(newMessageInfoList);
            }
            
        } else {
            //we've found the message, we set it over the old copy
            elements[pos] = messageInfo;
            //we do not need to recalculate short string, the message should
            // be the same except for the flags
            ((CanvasMessageItem)messageList.elementAt(pos)).message = messageInfo;
        }
        //Log.info(">>addMessage: " + Runtime.getRuntime().freeMemory());
        
        repaint();
    }
    
    /**
     * copy the objects in elements and messageList from the given position to
     * the given position
     * @param to the position to copy the objects to
     * @param from the position to copy the objects from
     */
    private void move(int to, int from) {
        
        elements[to] = elements[from];
        
        CanvasMessageItem mItem = (CanvasMessageItem) messageList.elementAt(from);
        mItem.setId(to);
        messageList.setElementAt(mItem, to);
        //messageList.elementAt()[to].setId(to);
        
        
    }
    /**
     * put the given messageinfo in the right position in
     * elements and messagelist
     *
     *@param to the position where to put the messageinfo
     *@param mInfo the messageInfo to be added
     */
    private void assign( int to, MessageInfo mInfo ) {
        elements[to] = mInfo;
        messageList.setElementAt(new CanvasMessageItem(mInfo, to),to);
    }
    
    
    
    /**
     * add a new messageInfo to the messagelist or overwrite the messageinfo
     * if exists
     *
     * @param message the message used to create the messaginfo info to be added
     */
    public void addMessage(Message message) {
        try {
            addMessage(new MessageInfo(message, new FolderInfo(
                    message.getParent().getName(), StoreFactory.getStore())));
        } catch (MailException ex) {
            ex.printStackTrace();
            Log.error("Error adding message " + message.getSubject()
            + "to message list");
        }
    }
    
    /**
     * @return the index of the messageinfo with the same messagid as
     * the given messageinfo, or -1 if not found.
     */
    public int search(String msgId) {
        int position = -1;
        for (int i =0; i< elements.length; i++) {
            if (getMessageInfoAt(i).messageId.equals(msgId)) {
                position = i;
                break;
            }
        }
        return position;
        
    }
    
    /**
     * handle the keypress events, should be overridden
     */
    protected void handleKey(int keyCode) {
    }
    
    /**
     *this method is called by implementation when the commands need to be
     * initialized (in the constructor), should be overridden. After this element
     * the commands referenced in addCommands, addItemCommands, removeItemcommands
     * must be not null
     */
    void initCommands() {
    }
    
    /**
     *this method is called by implementation when the generic commands need to be
     * added (in the constructor), should be overridden.
     */
    void addCommands() {
    }
    
    /**
     *this method is called by implementation when the item commands need to be
     * added (i.e. while setting a non empty elements array.  should be ovverridden
     */
    void removeItemCommands() {
    }
    
    /**
     *this method is called by implementation when the item commands need to be
     * removed (i.e. while setting an empty elements array). should be ovverridden
     */
    void addItemCommands() {
    }
    
    /**
     * paint the background
     */
    void paintBackground(Graphics graphics) {
        drawer.drawCanvasBackground(graphics, getWidth(), getHeight());
    }
    
    
    /**
     * remove the messageinfo in the given position from the visualized list.
     * This function will NOT delete the message from the folder
     *
     * @param position the position of the messageinfo to be removed
     */
    public void deleteMessageAt(int position) {
        
        //#ifdef low_mem
//#         UIController.freeMemory();      //XXX
        //#endif
        
        // Update elements.
        MessageInfo[] newElements = new MessageInfo[elements.length -1];
        for (int i=0, j=0, l=elements.length; i<l ; i++) {
            if (position != i) {
                newElements[j++] = (MessageInfo) elements[i];
            }
        }
        this.elements = newElements;
        
        // Update messageList.
        messageList.removeElementAt(position);
        for (int i = position; i< messageList.size(); i++) {
            ((CanvasMessageItem) messageList.elementAt(i)).setId(i);
        }
        
        // If the deleted message was the last, set the new last as
        // active.
        if (getActiveElement() >= elements.length) {
            setActiveElement(elements.length-1);
        }
        
        repaint();
    }
    
    /**
     * delete the message with the given msgID from the elements array, if present
     * this method does not delete the message from the rms!
     *
     * @param msgId the messageid of the message to be deleted
     */
    public void deleteMessage(String msgId) {
        int p = search(msgId);
        if ( p >= 0) {
            deleteMessageAt(p);
        }
    }
    
    /**
     * return the messageInfoList
     */
    public MessageInfo[] getMessageInfoList(){
        return (MessageInfo[])this.elements;
    }
    
    /**
     * return the messageInfo element at the given position
     * @param position the position of the element to be returned
     */
    public MessageInfo getMessageInfoAt(int position) {
        return ((MessageInfo)elements[position]);
    }
    
    
    
//------------------------ CanvasMessageItem ---------------------------------//
    
    
    /**
     * this class draws the message item
     */
    class CanvasMessageItem {
        
        
        MessageInfo message;
        private int id;
        private int lastw;
        private String date;
        
        /**
         * the subject that fits on a single line
         */
        private String shortSubject;
        
        /**
         * the address that fits on a single line
         */
        private String shortAddress;
        
        /**
         * the height of this element when not expanded
         */
        private int unexpanded_h;
        
        /**
         * the height of this element when expanded
         */
        private int expanded_h;
        
        
        /**
         * track if the last time we painted the item the read flag was
         * set or not (to recalculate the short strings if necessary)
         */
        private boolean lastReadFlag ;
        
        
        CanvasMessageItem(MessageInfo msgInfo, int id) {
            this.message = msgInfo;
            this.id = id;
            
        }
        
        private boolean isActive(){
            return id == getActiveElement();
        }
        
        /**
         * paint the messageitem
         */
        public int paint(Graphics graphics) {
            
            //we need to calculate the short strings
            int w = getWidth();
            int h = calculateHeight();
            calculateShortStrings();
            
            //we draw the background, image and date
            drawer.drawBackground(graphics,w,h,isActive());
            graphics.drawImage(Theme.getSTATE_IMAGES_VECTOR()[getFlagValue()],MARGIN,MARGIN,Graphics.TOP|Graphics.LEFT);
            drawer.drawString(getDate(),graphics,w-MARGIN,MARGIN,Graphics.TOP|Graphics.RIGHT,isActive());
            
            //if this is a read message we use normal font, otherwhise we use the bold font
            if (isRead()) {
                
                drawer.drawString(shortAddress, graphics, Theme.getIMAGE_PART_WIDTH()+
                        2*MARGIN, MARGIN, isActive());
            } else {
                drawer.drawBoldString(shortAddress, graphics, Theme.getIMAGE_PART_WIDTH()+
                        2*MARGIN, MARGIN, isActive());
            }
            //if expanded we draw the second line
            if (isExpanded()) {
                drawer.drawString(shortSubject, graphics, MARGIN, getUnexpandedh(), isActive());
            }
            return h;
        }
        
        
        /**
         * return the flag value used to identify the image in the image array
         */
        private int getFlagValue() {
            int flags=message.flags.getFlags();
            //we mask the flag to be sure we have a return value <32
            flags = (flags&0x0000001f);
            return flags;
        }
        
        /**
         * calculate the short version of address and subject and
         * store them in shortAddress and shortSubject
         */
        private void calculateShortStrings() {
            if (shortAddress == null || shortSubject == null || lastReadFlag != isRead() ) {
                int w = getWidth();
                int firstlineSpace =  w - 4*MARGIN - Theme.getIMAGE_PART_WIDTH() - font.stringWidth(getDate());
                int secondlineSpace = w - MARGIN;
                shortSubject=UiUtils.cutString(message.subject,secondlineSpace, font);
                String address = (isIncomingMessage()) ? message.from.getVisibleName() :
                    message.to.getVisibleName() ;
                
                if (isRead()) {
                    shortAddress=UiUtils.cutString(address, firstlineSpace, font);
                    lastReadFlag = true;
                } else {
                    shortAddress=UiUtils.cutString(address, firstlineSpace, boldFont);
                    lastReadFlag = false;
                }
                
                
            }
        }
        /**
         * @return true if message is in the inbox
         */
        private boolean isIncomingMessage() {
            return (Store.INBOX.equals(message.parent.getName()));
        }
        
        /**
         * @return true if the message has been opened
         */
        private boolean isRead() {
            return message.flags.isSet(MessageFlags.OPENED) ;
        }
        
        
        /**
         * return true if this item is expanded, false otherwise
         */
        public boolean isExpanded() {
            boolean ret;
            //#ifdef MessageItemAlwaysExpanded
//#             ret = true;
            //#else
            ret = isActive();
            //#endif
            return ret;
        }
        
        
        /**
         * compute height using information from the bean,
         * the drawer and the icon size
         */
        private int calculateHeight() {
            if (isExpanded()) {
                return getExpandedh();
            } else {
                return getUnexpandedh();
            }
        }
        
        /**
         * @retunr the height of this element not expanded
         */
        private int getUnexpandedh() {
            if (unexpanded_h==0) {
                unexpanded_h= MARGIN;
                unexpanded_h+=Math.max(
                        Math.max(font.getHeight(),boldFont.getHeight()), Theme.getIMAGE_PART_HEIGHT());
                //unselected_h+=MARGIN;
            }
            return unexpanded_h;
        }
        
        /**
         * @retunr the height of this element expanded
         */
        private int getExpandedh() {
            if (expanded_h==0) {
                expanded_h= getUnexpandedh();
                expanded_h+=font.getHeight();
                expanded_h+=MARGIN;
            }
            return expanded_h;
        }
        
        //  new Vector()
        /**
         * return the date as a formatted string
         */
        private String getDate() {
            if (date == null) {
                date = UiUtils.formatDate(message.received);
            }
            return date;
        }
        
        /**
         *
         */
        private void setId(int newId) {
            this.id = newId;
        }
    }
    
    
    boolean isVisible(int elementId) {
        return elements[elementId] != null;
        //return true;
    }
    
    boolean isActivable(int elementId) {
        return elements[elementId] != null;
    }
}
