/*
 * Copyright (C) 2006-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

/*
 * FunCanvasViewMessage.java
 */

package com.funambol.mailclient.ui.view;

import com.funambol.mail.MailException;
import com.funambol.mail.Message;
import com.funambol.mail.MessageFlags;
import com.funambol.mail.RMSStore;
import com.funambol.mail.Store;
import com.funambol.mailclient.cm.Contact;
import com.funambol.mailclient.loc.Localization;
import com.funambol.mailclient.sm.SyncClient;
import com.funambol.mailclient.ui.controller.UIController;
import com.funambol.mailclient.ui.utils.UiUtils;
import com.funambol.util.ChunkedString;
import com.funambol.util.Log;
import com.funambol.util.MailDateFormatter;
import com.funambol.util.StringUtil;
import java.util.Calendar;
import java.util.Date;
import java.util.TimeZone;
import java.util.Vector;
import javax.microedition.lcdui.Canvas;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;


public class FunCanvasViewMessage extends Canvas implements CommandListener,
        Runnable {
    /**
     * the message
     */
    private Message message;
    
    private Drawer drawer;
    
    private GraphicalTheme theme;
    
    private Font bodyFont;
    
    private static final int SCROLL_AMOUNT = 2;
    
    
    private static final int MARGIN=2;
    
    private static final int SCROLLBAR_WIDTH =4;
    /**
     * are we inside the header?
     */
    private boolean insideHeader = true;
    
    /**
     * are we inside the body?
     */
    private boolean insideBody = false;
    
    private BodyItem bodyItem;
    
    private boolean showHeader = true;
    
    private HeaderItem headerItem;
    
    private Command deleteCommand;
    
    private Command forwardCommand;
    
    private Command linksCommand;
    
    //private Command numbersCommand;
    
    private Command fullMessageCommand;
    
    private Command recipientsCommand;
    
    private Command replyCommand;
    
    private Command replyAllCommand;
    
    private Command saveSenderCommand;
    
    private Command markAsReadCommand;
    
    private Command followUpCommand;
    
    private Command editCommand;
    
    private Command sendCommand;
    
    private int NEXT_MESSAGE = 1;
    
    private int PREVIOUS_MESSAGE = -1;
    
    private String theTitle = Localization.getMessages().MESSAGE_VIEW_FORM_TITLE ;
    /**
     * creates a new FunCanvasViewMessage with given message
     */
    public FunCanvasViewMessage(Message message) {
        
        this.message = message;
        this.drawer = UIController.getDrawer();
        this.theme = drawer.getGraphicalTheme();
        addCommands();
        setCommandListener(this);
        bodyFont = theme.getDefaultFont();
        this.bodyItem = new BodyItem();
        this.headerItem = new HeaderItem();
        //setTitle(Localization.getMessages().MESSAGE_VIEW_FORM_TITLE);
        repaint();
    }
    
    /**
     * paint the viewmessagteitem
     */
    protected void paint(Graphics graphics) {
        drawer.drawCanvasBackground(graphics, getWidth(), getHeight());
        //drawer.drawString("ciao", graphics, 0,0,false);
        int h= paintTitle(graphics);
        h = headerItem.draw(graphics,getWidth(),h);
        bodyItem.draw(graphics,getWidth()- SCROLLBAR_WIDTH - MARGIN ,h);
        drawScrollBar(graphics, h);
    }
    
    
    public int paintTitle(Graphics g) {
        
        //   Log.debug("paint title!");
        
            /*
             *
             *    int titleheight = drawer.getGraphicalTheme().getTitleFont().getHeight();
        drawer.drawTitleBackground(graphics, getWidth(), titleheight);
        drawer.drawString(graphics,1,1, false);
             */
        
        
        String s;
        
        Font titleFont = drawer.getGraphicalTheme().getTitleFont();
        // keep this because otherwhise some motorola phones may throw a
        // nullpointer exception
        if (theTitle == null){
            //Log.debug("title null... setting to empty string");
            theTitle ="";
        }
        
        String shortTitle = UiUtils.cutString(theTitle,getWidth()-1,titleFont);
        drawer.drawTitleBackground(g,getWidth(), titleFont.getHeight());
        g.setFont(titleFont);
        g.setColor(drawer.getGraphicalTheme().getTitleColor());
        g.drawString(shortTitle,1,1,Graphics.TOP | Graphics.LEFT);
        g.drawString(theTitle,1,1,Graphics.TOP | Graphics.LEFT);
        return drawer.getGraphicalTheme().getTitleFont().getHeight()+1;
    }
    
    public void setTitle(String title) {
        
        //Log.error("setting title to " + title);
        
        //we have to ignore setTitle (null) because some motorola devices
        //call this method when building the canvas
        //TODO: use ifdef 
        if (title != null) {
            //      Log.debug("calls to setTitle (null): ignoring it");
            Log.error("setting title to " + title);
            theTitle = title;
        }
       
        //don't do a repaint here, or Nokia 6630 will crash.
        
    }
    
    public String getTitle() {
        return this.theTitle;
    }
    
    private void exitBody(int dir) {
        //  Log.debug("exit body");
        if (dir == Canvas.UP) {
            insideBody = false;
            enterHeader(dir);
        }
    }
    
    private void enterHeader(int dir) {
        // Log.debug("enter header");
        insideHeader = true;
        showHeader = true;
    }
    
    private void exitHeader(int dir) {
        //  Log.debug("exit header");
        if ( (dir == Canvas.DOWN) ) {
            insideHeader  = false;
            showHeader = false;
            enterBody(dir);
        }
        
    }
    
    
    /**
     * Handle the key navigation
     * @param keyCode is the key that has been pressed
     */
    protected void keyPressed(int keyCode) {
        
        int dir = getGameAction(keyCode);
        if (dir==Canvas.DOWN||dir==Canvas.UP) {
            
            if (insideBody) {
                //   Log.debug("sending traverse to body Item");
                bodyItem.traverse(dir);
            } else if (insideHeader) {
                // Log.debug("sending traverse to header Item");
                headerItem.traverse(dir);
            }
            repaint();
        } else if (dir==Canvas.RIGHT) {
            this.setTitle(Localization.getMessages().MESSAGE_VIEW_NEXT_MESSAGE);
            repaint();
            serviceRepaints();
            getNextMessage(NEXT_MESSAGE);
        } else if (dir==Canvas.LEFT) {
            this.setTitle(Localization.getMessages().MESSAGE_VIEW_PREVIOUS_MESSAGE);
            repaint();
            serviceRepaints();
            getNextMessage(PREVIOUS_MESSAGE);
        }
    }
    
    
    
    
    /**
     * Sets the correct message when left/rigth navigation is performed
     * @param shift is the
     */
    private void getNextMessage(int shift) {
        FunCanvasMessageList fcml = UIController.getMessageList(message);
        int msgNumber = fcml.getNumMessages();
        int messageToSet = fcml.getActiveElement()+shift;
        if (messageToSet>=msgNumber) {
            messageToSet=0;
            fcml.rewindList();
            shift=0;
        } else if (messageToSet<0) {
            messageToSet=msgNumber-1;
            shift=fcml.elements.length;
        }
        
        if (Math.abs(shift)>1) {
            for (int i=0; i<shift; i++) {
                fcml.scroll(1);
                fcml.repaint();
            }
        } else {
            fcml.scroll(shift);
        }
        fcml.setActiveElement(messageToSet);
        cleanup();
        UIController.showViewMessageScreen(
                fcml, fcml.getSelectedMessageInfo().getMessage());
        //repaint();
    }
    
    
    /**
     * this method creates a vector of strings that can fit
     * on a single line of screenWidth pixel with given font.
     */
    public String[] getStringArray(String text, int screenWidth, Font font) {
        // Log.debug(this, "GetStingArray() " + text.substring(0,10));
        
        Vector lines = new Vector();
        String [] ret;
        
        ChunkedString csLine = new ChunkedString(text);
        ChunkedString csWord;
        String[] lineSep = {"\r\n", "\n\r", "\r", "\n" };
        String[] wordSep = {" ", "\t"};
        
        
        //ChunkedString line;
        StringBuffer wordStringBuffer = new StringBuffer();
        String word;
        StringBuffer myLine=new StringBuffer();
        int widthLeft;
        while ( (csWord=csLine.getNextChunk(lineSep)) != null ) {
            //csWord = new ChunkedString(line);
            widthLeft = screenWidth;
            myLine.setLength(0);
            
            // get the words
            while ( (word = csWord.getNextString(wordSep)) != null ) {
                wordStringBuffer = new StringBuffer(word);
                
                if ( myLine.length() != 0 ) {
                    //if this is not the first word of the line we add a space
                    wordStringBuffer.insert(0," ");
                }
                int width = font.stringWidth(wordStringBuffer.toString());
                //if it fits inside the line
                if (width < widthLeft) {
                    //we append the word at the line and update widthleft
                    myLine.append(wordStringBuffer.toString());
                    widthLeft -=width;
                } else {
                    //if we've a space we can cut it out since this word
                    // will go to the next line
                    if (wordStringBuffer.charAt(0) == ' ') {
                        wordStringBuffer.deleteCharAt(0);
                    }
                    width = font.stringWidth(wordStringBuffer.toString());
                    //if can go on a new line
                    if ( width < screenWidth ) {
                        //we put it on a new line
                        lines.addElement(myLine.toString());
                        myLine.setLength(0);
                        myLine.append(wordStringBuffer.toString());
                        widthLeft = screenWidth - width;
                        continue;
                    } else {
                        int start = 0;
                        int end = 0;
                        int length = wordStringBuffer.length();
                        //String aString = myLine.toString();
                        while (start < length) {
                            // check length of the substring, adding a single char each time
                            for (int w = 0; w < widthLeft && end < length; end++) {
                                w = font.substringWidth( wordStringBuffer.toString(), start, (end - start) +1 );
                            }
                            myLine.append(wordStringBuffer.toString().substring(start,end));
                            if (end < length) {
                                lines.addElement(myLine.toString());
                                myLine.setLength(0);
                                widthLeft = screenWidth;
                            } else {
                                widthLeft = screenWidth - font.stringWidth(myLine.toString());
                            }
                            start = end;
                        }
                    }
                }
            }
            lines.addElement(myLine.toString());
        }
        ret = new String[lines.size()];
        //        Log.debug(this, "created vector, copying it");
        lines.copyInto(ret);
        lines=null;
        //       Log.debug(this, "returning string array");
        return ret;
    }
    
    private void addCommands() {
        addCommand(UIController.backCommand);
        deleteCommand =new Command(Localization.getMessages().DELETE_COMMAND_LABEL,
                UIController.COMMAND_TYPE,10);
        forwardCommand=new Command(Localization.getMessages().MVF_FORWARD_COMMAND,UIController.COMMAND_TYPE,25);
        linksCommand=new Command(Localization.getMessages().MVF_LINKS_COMMAND, UIController.COMMAND_TYPE,55);
        //numbersCommand=new Command(Localization.getMessages().MVF_NUMBERS_COMMAND, UIController.COMMAND_TYPE,65);
        fullMessageCommand=new Command(
                Localization.getMessages().MVF_FULL_MSG_CMD,
                UIController.COMMAND_TYPE, 1);
        
        
        //this commands are always present
        this.addCommand(UIController.backCommand);
        this.addCommand(forwardCommand);
        this.addCommand(linksCommand);
        //this.addCommand(numbersCommand);
        
        // This is present if the message is incomplete.
        if(message.getFlags().isSet(MessageFlags.PARTIAL)) {
            this.addCommand(fullMessageCommand);
        }
        
        //this are present if the message is incoming
        if (UiUtils.isIncoming(message)) {
            replyCommand=new Command(Localization.getMessages().MVF_REPLY_COMMAND,UIController.COMMAND_TYPE,9);
            replyAllCommand=new Command(Localization.getMessages().MVF_REPLY_ALL_COMMAND, UIController.COMMAND_TYPE, 15);
            //flagCommand=new Command("Flag",Command.SCREEN,4);
            saveSenderCommand=new Command(Localization.getMessages().MVF_SAVE_SENDER_COMMAND, UIController.COMMAND_TYPE,45);
            recipientsCommand=new Command(Localization.getMessages().MVF_RECIPIENTS_COMMAND,UIController.COMMAND_TYPE,75);
            
            setMarkAsReadCommand();
            setFollowUpCommand();
            
            this.addCommand(deleteCommand);
            this.addCommand(replyCommand);
            this.addCommand(replyAllCommand);
            //this.addCommand(flagCommand);
            this.addCommand(saveSenderCommand);
            this.addCommand(recipientsCommand);
        } else if (UiUtils.isOutgoing(message)) {
            //outbox or draft
            sendCommand=new Command(Localization.getMessages().SEND_COMMAND_LABEL, UIController.COMMAND_TYPE, 0);
            editCommand=new Command(Localization.getMessages().EDIT_COMMAND_LABEL, UIController.COMMAND_TYPE, 1);
            this.addCommand(sendCommand);
            this.addCommand(editCommand);
        } else if (UiUtils.isSent(message)) {
            //sendCommand=new Command(Localization.getMessages("MVF_ResendCommand"),UIController.COMMAND_TYPE, 0);
            editCommand=new Command(Localization.getMessages().EDIT_COMMAND_LABEL, UIController.COMMAND_TYPE, 1);
            //this.addCommand(sendCommand);
            this.addCommand(editCommand);
        }
        this.setCommandListener(this);
        
    }
    
    private void setMarkAsReadCommand() {
        
        removeCommand(markAsReadCommand);
        if(message.getFlags().isSet(MessageFlags.OPENED))
            markAsReadCommand = new Command(Localization.getMessages().UNREAD_COMMAND_LABEL, UIController.COMMAND_TYPE,35);
        else
            markAsReadCommand = new Command(Localization.getMessages().READ_COMMAND_LABEL, UIController.COMMAND_TYPE,35);
        
        this.addCommand(markAsReadCommand);
    }
    
    private void setFollowUpCommand() {
        
        removeCommand(followUpCommand);
        if(message.getFlags().isSet(MessageFlags.FLAGGED))
            followUpCommand = new Command(Localization.getMessages().UNFLAG_COMMAND_LABEL, UIController.COMMAND_TYPE,30);
        else
            followUpCommand = new Command(Localization.getMessages().FLAG_COMMAND_LABEL, UIController.COMMAND_TYPE,30);
        
        this.addCommand(followUpCommand);
    }
    
    /**
     *
     */
    public void commandAction(Command command, Displayable displayable) {
        
        if (command==UIController.backCommand) {
            
            try {
                if (!SyncClient.isBusy()) {
                    UIController.getInboxMessageList().resetTitle();
                }
                UIController.getInboxMessageList().activate(
                        UIController.getInboxMessageList().getActiveElement());
                
                cleanup();    // FIXME
                UIController.showInboxScreen();
                
                //UIController.showBackScreen();
                
            } catch (Exception ex) {
                ex.printStackTrace();
                Log.error("ViewMessage: "+ex);
            }
        } else if (command==replyCommand) {
            UIController.replyMessage(message);
            cleanup();    // FIXME
        } else if (command==forwardCommand) {
            UIController.forwardMessage(message);
            cleanup();    // FIXME
        } else if (command==deleteCommand) {
            UIController.deleteMessage(message);
            cleanup();    // FIXME
            UIController.showInboxScreen();
        } else if (command==saveSenderCommand) {
            if (UIController.checkContactListFull()) {
                UIController.showErrorAlert(
                        Localization.getMessages().AB_LIMIT_ERROR_MSG);
                return;
            }
            Contact contact;
            try {
                if (message.getFrom() != null) {
                    contact = new Contact(
                            message.getFrom().getName(),
                            message.getFrom().getEmail());
                    UIController.showAddContactScreen(
                            UIController.display.getCurrent(),contact);
                } else {
                    UIController.showAlert(Localization.getMessages().FROM_IS_NULL);
                }
                
            } catch (Exception ex) {
                ex.printStackTrace();
                Log.error(this, "Mail Exception while adding Contact" + ex);
                UIController.showAddContactScreen(UIController.display.getCurrent());
                UIController.showErrorAlert(
                        Localization.getMessages().MESSAGE_VIEW_UNABLE_TO_CREATE_CONTACT);
            }
            
        }  else if (command==linksCommand) {
            UIController.showLinkList(UIController.display.getCurrent(), message.getTextContent());
        }
        /*else if (command==numbersCommand ){
            UIController.showPhoneNumberList(UIController.display.getCurrent(), message.getTextContent());
        }*/
        else if (command==recipientsCommand) {
            UIController.showRecipientsList(UIController.display.getCurrent(), message);
        } else if (command==editCommand) {
            message.setMessageId(message.createUniqueMessageIDValue());
            UIController.updateComposeMessageScreen(message,"");
            UIController.showComposeMessageScreen(UIController.display.getCurrent(),UIController.getComposeMessageScreen().body);
            //XXX
        } else if (command==replyAllCommand) {
            UIController.replyAll(message);
            cleanup();    // FIXME
        } else if (command == markAsReadCommand) {
            UIController.changeReadFlag(message);
            setMarkAsReadCommand();
        } else if (command == followUpCommand) {
            UIController.changeFollowUpFlag(message);
            setFollowUpCommand();
        } else if (command==fullMessageCommand) {
            this.removeCommand(fullMessageCommand);
            this.setTitle(Localization.getMessages().MVF_FULL_MESSAGE_LOADING);
            UIController.getThreadPool().startThread(this);
        } else {
            Log.info("MessageViewForm: command not implemented");
        }
    }
    
    private void enterBody(int dir) {
        insideBody = true;
    }
    
    protected void keyRepeated(int keyCode) {
        keyPressed(keyCode);
    }
    
    
    private void drawScrollBar(Graphics g, int h) {
        
        int maxHeight = getHeight() - h;
        
        
        int barheight = (int) ( ( ( bodyItem.getLastLine()*100 -
                bodyItem.getCurrentLine()*100 ) /bodyItem.getNumLines())
                * maxHeight)/100;
        int barposition = (int) (h+((100*  bodyItem.getCurrentLine()* maxHeight
                / bodyItem.getNumLines() ) )/100);
        
        g.setColor(theme.getForegroundColor());
        g.drawRect(getWidth()- SCROLLBAR_WIDTH, h, SCROLLBAR_WIDTH, maxHeight);
        g.fillRect(getWidth() - SCROLLBAR_WIDTH,barposition, SCROLLBAR_WIDTH,
                barheight);
    }
    
    
    /**
     * get the full message
     */
    public void run() {
        this.removeCommand(fullMessageCommand);
        setMessage(UIController.getFullMessage(message));
    }
    
    /**
     * FIXME: this is just a patch. Needs to find why the viewMessage is
     * not collected by the gc.
     */
    private void cleanup() {
        this.message=null;
        this.bodyItem=null;
        this.headerItem=null;
    }
    
    private void setMessage(Message fullmsg) {
        FunCanvasMessageList fcml = UIController.getMessageList(message);
        if(fullmsg == null ) {
            Log.info("The server didn't send the full message.");
            return;
        }
        
        if ( bodyItem == null || headerItem == null ) {
            Log.info("bodyItem == null || headerItem == null ");
            return;
        }
        
        try {
            fullmsg.setParent(message.getParent());
            //Fix for message flags - see 2772 bug
            fullmsg.setFlags(message.getFlags());
            message = fullmsg;
            bodyItem.setMessage(message);
            headerItem.setMessage(message);
            this.setTitle(
                    (message.getFrom()==null)?"":message.getFrom().getVisibleName());
        } catch(MailException e) {
            this.setTitle(Localization.getMessages().MESSAGE_VIEW_FORM_TITLE);
            Log.error("Error getting VisibleName - " + e.getMessage());
            e.printStackTrace();
        } finally {
            this.removeCommand(fullMessageCommand);
            setTitle("Message " + (fcml.getActiveElement()+1)
            + " of " + fcml.getNumMessages());
        }
    }
    
    
    
//------------------------------BODYITEM CLASS ---------------------------//
    
    /**
     * This class handle the visualization of the body of the message.
     */
    class BodyItem {
        private int currentLine;
        private int maxLineIndex;
        private int fontHeight;
        private Vector textStringsVector;
        private String[] textStrings;
        private int bodyLastWidth = -1;
        private int screenHeight;
        
        
        private int lastline;
        
        public BodyItem() {
            fontHeight = bodyFont.getHeight();
            screenHeight = getHeight();
        }
        
        public void traverse(int dir) {
    /*
            for future implementation of internal navigation
     
            if( dir == Canvas.RIGHT) {
                toNextMessage();
                return true;
            } else if (dir == Canvas.LEFT) {
                toPreviousMessage();
                return true;
            }
     */
            if (insideBody) {
                if (dir == Canvas.DOWN) {
                    scroll(Canvas.DOWN);
                    if ( isAtBottom() ) {
                        
                        currentLine =    Math.max( maxLineIndex + 2 -
                                screenHeight / fontHeight, 0);
                        //visRect_inout[1] = calculateY(viewportHeight);
                        exitBody(dir);
                    }
                } else if (dir == Canvas.UP) {
                    scroll(Canvas.UP);
                    if ( isAtTop() ) {
                        currentLine = 0;
                        exitBody(dir);
                    }
                }
            }
        }
        
        public void setMessage(Message fullmsg) {
            //we add a \n to the messagebody for nicer visualization
            textStrings=getStringArray(fullmsg.getTextContent()+" \r\n",bodyLastWidth, bodyFont);
            maxLineIndex = Math.max(textStrings.length -1, 1);
            //       Log.debug (this, "new max line index " + maxLineIndex);
            
        }
        private void scroll(int dir) {
            if ( dir == Canvas.DOWN ) {
                currentLine += SCROLL_AMOUNT ;
            } else if (dir == Canvas.UP) {
                currentLine -= SCROLL_AMOUNT;
            }
            // Log.info ("Currentline: " + currentLine);
        }
        
        /**
         * draw the element with given width starting at given height
         *
         *@return the height to start painting the next item
         */
        public int draw(Graphics g, int w, int h) {
            //  Log.debug(this, "initializing body vectors");
            initializeBodyStringVectors(w-3*MARGIN);
            //   Log.debug(this, " body vectors initialized");
            
            //  drawer.drawBackground(g,w,h,false);
            
            //we draw the lines
            int i;
            for (i= currentLine; i<=currentLine + screenHeight/fontHeight; i++ ) {
                if (i<textStrings.length) {
                    drawer.drawString(textStrings[i], g, MARGIN, h, false);
                    h+=fontHeight;
                    
                } else {
                    break;
                }
            }
            lastline = i;
            
            return h ;
        }
        
        private void initializeBodyStringVectors(int w){
            if (bodyLastWidth!=w) {
                
                //we add a \n to have nicer visualization
                textStrings=getStringArray(message.getTextContent()+" \n",w,
                        drawer.getGraphicalTheme().getDefaultFont());
                bodyLastWidth = w;
                maxLineIndex = Math.max(textStrings.length -1,1);
            }
        }
        private boolean isAtBottom() {
            return (currentLine -2 + screenHeight/fontHeight >=maxLineIndex);
        }
        
        private boolean isAtTop() {
            return ( currentLine <= 0 );
        }
        
        protected int getCurrentLine() {
            return currentLine;
        }
        
        protected int getLastLine() {
            return lastline;
        }
        
        protected int getNumLines() {
            return maxLineIndex;
        }
        
        
        
    }
    
    //------------------------------- HEADERITEM CLASS -----------------------//
    
    /**
     * This class handle the visualization of the body of the message.
     */
    class HeaderItem {
        private int headerLastWidth=-1;
        private boolean expanded;
        
        /**
         * message fields
         */
        private String from="";
        private String subject="";
        private String to="";
        private String date="";
        private String cc="";
        private String bcc="";
        
        /**
         * String constants
         */
        private final String FROM=Localization.getMessages().MVHI_FROM_LABEL;
        private final String SUBJECT=Localization.getMessages().MVHI_SUBJ_LABEL;
        private final String TO=Localization.getMessages().TO_LABEL;
        private final String DATE=Localization.getMessages().MVHI_ON_LABEL;
        private final String CC=Localization.getMessages().CC_LABEL;
        private final String BCC=Localization.getMessages().BCC_LABEL;
        
        
        
        /**
         * vector to store lines
         */
        private String [] fromStrings;
        private String [] subjectStrings;
        private String  toStrings;
        private String [] dateStrings;
        private String  ccStrings;
        private String  bccStrings;
        
        private final int BORDER=2;
        
        private Font fromFont= theme.getSmallFont();
        private Font toFont= theme.getSmallFont();
        private Font subjectFont = theme.getSmallFont();
        private Font hiddenFieldsFont = theme.getSmallFont();
        private Font visibleFieldsFont = theme.getSmallFont();
        private int subjectColor= theme.getHighlightedForegroundColor();
        private int hiddenFieldsColor=theme.getHighlightedForegroundColor();
        private int visibleFieldsColor=theme.getHighlightedForegroundColor();
        
        public HeaderItem() {
            //filling fields..
            setMessage(message);
            
        }
        
        private void setFonts() {
          /*  if (UiUtils.isIncoming(message) ) {
                fromFont = theme.getDefaultFont();
                toFont = theme.getSmallFont();
            } else {
                fromFont = theme.getSmallFont();
                toFont = theme.getDefaultFont();
            }
           */
            subjectFont = theme.getSmallBoldFont();
            subjectColor = theme.getHighlightedForegroundColor();
        }
        
        private void setMessage(Message msg) {
            try {
                if (message.getFrom()!=null) {
                    from=message.getFrom().getVisibleName();
                } else {
                    from ="";
                }
            } catch (MailException ex) {
                Log.error(this,"error getting from field");
                ex.printStackTrace();
            }
            
            try {
                for (int i=0; i<message.getTo().length; i++) {
                    to+= (i==0) ? "" : ", ";
                    to+=message.getTo()[i].getVisibleName().trim();
                }
            } catch (Exception ex) {
                ex.printStackTrace();
                Log.error(this, "error fetching to addresses");
            }
            
            try {
                for (int i=0; i<message.getCc().length; i++) {
                    cc+= (i==0) ? "" : ", ";
                    cc+=message.getCc()[i].getVisibleName().trim();
                }
            } catch (Exception ex) {
                ex.printStackTrace();
                Log.error(this, "error fetching cc addresses");
            }
            
            try {
                for (int i=0; i<message.getBcc().length; i++) {
                    bcc+= (i==0) ? "" : ", ";
                    bcc+=message.getBcc()[i].getVisibleName().trim();
                }
            } catch (Exception ex) {
                ex.printStackTrace();
                Log.error(this, "error fetching Bcc addresses");
            }
            
            Date dateObj = message.getSentDate();
            // Calendar calendar = Calendar.getInstance();
            // calendar.setTime(dateObj);
            // calendar.setTimeZone(TimeZone.getDefault());
            
            // this is only for visualization purposes
            //TODO: check if the variables in formatLocalTime are really used
            
            //date = MailDateFormatter.formatLocalTime(calendar.getTime());
            date = MailDateFormatter.formatLocalTime(dateObj);
            subject=message.getSubject();
            
            setFonts();
            
            
            
        }
        
        
        /**
         *a traverse-like method that updates the state of this item
         */
        public void traverse(int dir) {
            
            // Log.debug("traverse header Item, insideheader = " + insideHeader);
            if (insideHeader) {
                if ( (dir==Canvas.UP) ) {
                    //  Log.debug("traverse up");
                    if(!expanded) {
                        expand();
                    }
                    //else expand();
                } else if ( (dir == Canvas.DOWN) ) {
                    //   Log.debug("traverse down");
                    if (expanded) {
                        shrink();
                    } else {
                        exitHeader(dir);
                    }
                }
            }
        }
        private void shrink() {
            //  Log.debug("shrink()");
            expanded=false;
        }
        
        private void expand() {
            //   Log.debug("expand()");
            expanded=true;
        }
        /**
         * @return the estimated height for this item, used to draw the
         * background etc.
         */
        protected int calculateHeight(int width) {
            
            if (!showHeader) return (0);
            
            //TODO: check why sometimes passed width is 0
            if (width==-1 || width==0) {
                initializeHeaderStringVectors(width-4*BORDER);
            } else
                initializeHeaderStringVectors(width-4*BORDER);
            
            int  h=subjectFont.getHeight()* subjectStrings.length;
            if (UiUtils.isIncoming(message)) {
                h+=visibleFieldsFont.getHeight()* fromStrings.length;
            } else {
                if (toStrings!= null)
                    h+=visibleFieldsFont.getHeight();
            }
            
            if (expanded) {
                if (UiUtils.isIncoming(message)) {
                    h+=hiddenFieldsFont.getHeight();
                }else {
                    h+=hiddenFieldsFont.getHeight()* fromStrings.length;
                }
                
                h+=hiddenFieldsFont.getHeight()*dateStrings.length;
                if (ccStrings!= null) {
                    h+=hiddenFieldsFont.getHeight();
                }
                if (bccStrings!=null) {
                    h+=hiddenFieldsFont.getHeight();
                }
            }
            h+=4*BORDER;
            return h;
        }
        
        private void initializeHeaderStringVectors(int w){
            if (headerLastWidth!=w) {
                int width=w-4*BORDER;
                if (from.equals("")) {
                    fromStrings=new String[0];
                } else {
                    fromStrings=getStringArray(FROM + " " + from, width, fromFont);
                }
                subjectStrings=getStringArray(SUBJECT+" " +subject,width, subjectFont);
                if (to.length()!=0) {
                    toStrings = UiUtils.cutString(TO+" "+to, width,toFont);//getStringArray(TO+" "+to, width,toFont);
                } /*else {
                    toStrings= new String();
                }*/
                
                dateStrings=getStringArray(DATE+" " +date,width,hiddenFieldsFont);
                if (cc.length()!=0) {
                    ccStrings = UiUtils.cutString(CC+" "+cc, width,hiddenFieldsFont);//getStringArray(CC+" " +cc, width,hiddenFieldsFont);
                } /*else {
                    ccStrings=new String();
                }*/
                if (bcc.length()!=0) {
                    bccStrings = UiUtils.cutString(BCC+" "+bcc, width,hiddenFieldsFont);//getStringArray(CC+" " +cc, width,hiddenFieldsFont);
                } /*else {
                    bccStrings=new String();
                }*/
                headerLastWidth=w;
            }
            
        }
        
        
        /**
         * paint function. it draws the strings ecc.
         *h it's the current graphics height to print at,
         *it's modified by this function to return the next y where content
         *can be drawn.
         *@return the h + the height drawn by the item
         */
        protected int draw(Graphics graphics, int w, int h) {
            //graphics.getFont().
            //   int oldCol=graphics.getColor();
            
            //   Log.debug("drawing headerItem");
            
            
            if (!showHeader) {
                return h;
            }
            int inith = h;
            initializeHeaderStringVectors(w-2*BORDER);
            int height = calculateHeight(w);
            
            
            //we need to translate the graphics, because the drawer.drawbackground
            //mathod starts at 0,0
           /* graphics.translate(MARGIN, h);
            Log.debug("drawing background");
            drawer.drawBackground(graphics, w, h, true);
            graphics.translate(-MARGIN, -h);
            */
            
            
            graphics.setColor(theme.getHighlightedBackgroundColor());
            //TODO: use drawer and optimize!
            graphics.fillRoundRect(BORDER,h,w-2*BORDER,height,10,10);
            
            graphics.setColor(theme.getHighlightedBorderColor());
            graphics.drawRoundRect(BORDER,h,w-2*BORDER,height,10,10);
            
            h+=2*BORDER;
            
            //we set the font to draw the "from: address" line and we draw it
            //graphics.getDisplayColor()
            
            //if expanded we draw to, cc, bbc & date fields
            if (expanded){
                if (UiUtils.isIncoming(message)) {
                    h = paintToField(graphics, h);
                } else {
                    h = paintFromField(graphics, h);
                }
                h = paintCcField(graphics, h);
                h = paintBccField(graphics, h);
                h = paintDateField(graphics, h);
            }
            
            if (UiUtils.isIncoming(message)) {
                h=paintFromField(graphics, h);
            } else {
                h= paintToField(graphics,h);
            }
            h=paintSubjectField(graphics, h);
            
            
            graphics.setColor(theme.getHighlightedForegroundColor());
            if(!expanded) {
                //a small arrow up to let the user know we have something hidden that
                // can be discovered pressing the up key
                graphics.fillTriangle(w-9-BORDER,inith+8+BORDER,w-6-BORDER,inith+1+BORDER,w-2-BORDER,inith+8+BORDER);
            } else {
                //small arrow down
                graphics.fillTriangle(w-9-BORDER,inith+1+BORDER,w-6-BORDER,inith+8+BORDER,w-2-BORDER,inith+1+BORDER);
            }
            return height+ inith;
        }
        
        private int paintSubjectField(final Graphics graphics, int h) {
            //we set the font to draw the subject line
            graphics.setFont(subjectFont);
            graphics.setColor(subjectColor);
            //we draw the lines
            for (int i =0; i<subjectStrings.length; i++) {
                graphics.drawString(subjectStrings[i],2*BORDER,h,graphics.TOP|graphics.LEFT);
                h+=subjectFont.getHeight();
            }
            return h;
        }
        
        private int paintDateField(final Graphics graphics, int h) {
            graphics.setFont(hiddenFieldsFont);
            graphics.setColor(hiddenFieldsColor);
            for (int i = 0; i< dateStrings.length; i++) {
                graphics.drawString(dateStrings[i],2*BORDER,h,graphics.TOP|graphics.LEFT);
                h+=hiddenFieldsFont.getHeight();
            }
            return h;
        }
        
        private int paintBccField(final Graphics graphics, int h) {
            if (bccStrings == null) {
                return h;
            }
            graphics.setFont(hiddenFieldsFont);
            graphics.setColor(hiddenFieldsColor);
            
            graphics.drawString(bccStrings,2*BORDER,h,graphics.TOP|graphics.LEFT);
            h+=hiddenFieldsFont.getHeight();
            
            return h;
            
        }
        
        private int paintCcField(final Graphics graphics, int h) {
            if (ccStrings == null) {
                return h;
            }
            
            graphics.setFont(hiddenFieldsFont);
            graphics.setColor(hiddenFieldsColor);
            
            graphics.drawString(ccStrings,2*BORDER,h,graphics.TOP|graphics.LEFT);
            h+=hiddenFieldsFont.getHeight();
            
            return h;
        }
        
        private int paintFromField(final Graphics graphics, int h) {
            
            graphics.setColor(theme.getHighlightedForegroundColor());
            graphics.setFont(fromFont);
            for (int i = 0; i<fromStrings.length; i++) {
                //drawer.drawString(fromStrings[i], graphics, 2*BORDER,h,true);
                graphics.drawString( fromStrings[i],2*MARGIN,h,graphics.TOP|graphics.LEFT);
                h+=fromFont.getHeight();
            }
            return h;
        }
        
        private int paintToField(final Graphics graphics, int h) {
            if (toStrings==null) {
                return h;
            }
            graphics.setColor(theme.getHighlightedForegroundColor());
            graphics.setFont(toFont);
            
            graphics.drawString(toStrings,2*MARGIN,h,graphics.TOP|graphics.LEFT);
            h+=toFont.getHeight();
            return h;
            
        }
        
    }
}
