/*
 * Copyright (C) 2006-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package com.funambol.mailclient.loc;

/**
 * This class contains the localization support for the mail client.
 * The derived classes of Localization must contain all the messages
 * used by the client as public strings.
 * See the EnglishLocalization implementation for a guideline on how
 * to implement your own.
 */

/**
 * Basic localization support class.
 * This class can be adapted to support different languages.
 */
public class Localization {
    
    private static LocalizedMessages messages = null;

    private Localization() {
    }

    /**
     * Return the current locale or get the default one.
     * This method can be changed to return different locales depending on
     * the platform language. Currently supports only English.
     */
    public static LocalizedMessages getMessages() {
        if (messages == null) {
            messages = new LocalizedMessages();
        }
        return messages;
    }

    /**
     * Set a new localized message set.
     * Users of the class needs just to derive from LocalizedMessages,
     * overriding all the message strings. If a messages is not translated
     * the default one will be used.
     */
    public static void setMessages(LocalizedMessages msgs) {
        messages = msgs;
    }

}

